<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns="http://www.w3.org/1999/xhtml"
                              xmlns:html="http://www.w3.org/1999/xhtml"
                              xmlns:wwgloss="urn:WebWorks-Glossary-Schema"
                              xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                              xmlns:msxsl="urn:schemas-microsoft-com:xslt"
                              xmlns:wwsplits="urn:WebWorks-Engine-Splits-Schema"
                              xmlns:wwtoc="urn:WebWorks-Engine-TOC-Schema"
                              xmlns:wwlinks="urn:WebWorks-Engine-Links-Schema"
                              xmlns:wwmode="urn:WebWorks-Engine-Mode"
                              xmlns:wwfiles="urn:WebWorks-Engine-Files-Schema"
                              xmlns:wwdoc="urn:WebWorks-Document-Schema"
                              xmlns:wwbehaviors="urn:WebWorks-Behaviors-Schema"
                              xmlns:wwproject="urn:WebWorks-Publish-Project"
                              xmlns:wwpage="urn:WebWorks-Page-Template-Schema"
                              xmlns:wwlocale="urn:WebWorks-Locale-Schema"
                              xmlns:wwprogress="urn:WebWorks-XSLT-Extension-Progress"
                              xmlns:wwlog="urn:WebWorks-XSLT-Extension-Log"
                              xmlns:wwfilesystem="urn:WebWorks-XSLT-Extension-FileSystem"
                              xmlns:wwuri="urn:WebWorks-XSLT-Extension-URI"
                              xmlns:wwstring="urn:WebWorks-XSLT-Extension-StringUtilities"
                              xmlns:wwfilesext="urn:WebWorks-XSLT-Extension-Files"
                              xmlns:wwprojext="urn:WebWorks-XSLT-Extension-Project"
                              xmlns:wwexsldoc="urn:WebWorks-XSLT-Extension-Document"
                              exclude-result-prefixes="wwgloss xsl msxsl wwsplits wwtoc wwlinks wwmode wwfiles wwdoc wwbehaviors wwproject wwpage wwlocale wwprogress wwlog wwfilesystem wwuri wwstring wwfilesext wwprojext wwexsldoc"
>

 <xsl:param name="GlobalInput" />
 <xsl:param name="GlobalPipelineName" />
 <xsl:param name="GlobalProject" />
 <xsl:param name="GlobalFiles" />
 <xsl:param name="ParameterDependsType" />
 <xsl:param name="ParameterSplitsType" />
 <xsl:param name="ParameterType" />
 <xsl:param name="ParameterSplitFileType" />
 <xsl:param name="ParameterCategory" />
 <xsl:param name="ParameterUse" />
 <xsl:param name="ParameterDeploy" />

 <xsl:output method="xml" indent="yes" encoding="UTF-8" />
 <xsl:namespace-alias stylesheet-prefix="html" result-prefix="#default" />
 <xsl:strip-space elements="*" />

 <xsl:key name="wwfiles-files-by-type" match="wwfiles:File" use="@type" />
 <xsl:key name="wwfiles-files-by-groupid-type" match="wwfiles:File" use="concat(@groupID, ':', @type)" />
 <xsl:key name="wwsplits-files-by-type" match="wwsplits:File" use="@type" />

 <xsl:variable name="GlobalActionChecksum">
  <xsl:variable name="VarTransformChecksums">
   <xsl:value-of select="concat(wwuri:AsFilePath('wwtransform:self'), ':', wwfilesystem:GetChecksum(wwuri:AsFilePath('wwtransform:self')))" />
  </xsl:variable>
  <xsl:value-of select="wwstring:MD5Checksum($VarTransformChecksums)" />
 </xsl:variable>

 <xsl:template match="/">
  <wwfiles:Files version="1.0">

   <xsl:if test="wwprojext:GetFormatSetting('enable-glossary-tab', 'false') = 'true'">

    <xsl:variable name="VarProjectGroups" select="$GlobalProject/wwproject:Project/wwproject:Groups/wwproject:Group" />
    <xsl:value-of select="wwprogress:Start(count($VarProjectGroups))" />
    <xsl:for-each select="$VarProjectGroups">
     <xsl:variable name="VarProjectGroup" select="." />

     <xsl:value-of select="wwprogress:Start(1)" />
     <xsl:for-each select="$GlobalFiles[1]">
      <!-- Get output directory prefix -->
      <!--                             -->
      <xsl:variable name="VarFilesSplits" select="key('wwfiles-files-by-groupid-type', concat($VarProjectGroup/@GroupID, ':', $ParameterSplitsType))" />
      <xsl:variable name="VarSplits" select="wwexsldoc:LoadXMLWithoutResolver($VarFilesSplits/@path)" />
      <xsl:for-each select="$VarSplits[1]">
       <xsl:variable name="VarGlossaryDirSplit" select="key('wwsplits-files-by-type', $ParameterSplitFileType)" />
       <xsl:variable name="VarGlossaryDir" select="$VarGlossaryDirSplit/@path" />

       <xsl:for-each select="$GlobalFiles[1]">

        <!-- Get group merged glossary -->
        <!--                           -->
        <xsl:variable name="VarFilesGlossaryMerge" select="key('wwfiles-files-by-groupid-type', concat($VarProjectGroup/@GroupID, ':', $ParameterDependsType))" />
        <xsl:variable name="VarGlossaryMerge" select="wwexsldoc:LoadXMLWithoutResolver($VarFilesGlossaryMerge/@path)" />
        <xsl:variable name="VarGlossaryTerms" select="$VarGlossaryMerge/wwgloss:Glossary/wwgloss:Term" />

        <xsl:value-of select="wwprogress:Start(count($VarGlossaryTerms))" />
        <xsl:for-each select="$VarGlossaryTerms">
         <xsl:variable name="VarGlossaryTerm" select="." />

         <xsl:value-of select="wwprogress:Start(1)" />

         <!-- Path -->
         <!--      -->
         <xsl:variable name="VarPath" select="wwfilesystem:Combine($VarGlossaryDir, concat($VarGlossaryTerm/@value, '.html'))" />
         <xsl:variable name="VarUpToDate" select="wwfilesext:UpToDate($VarPath, $GlobalProject/wwproject:Project/@ChangeID, $VarProjectGroup/@GroupID, '', $GlobalActionChecksum)" />
         <xsl:if test="not($VarUpToDate)">
          <xsl:variable name="VarResultAsXML">
           <html:html>
            <html:head>
             <html:title><xsl:value-of select="$VarGlossaryTerm/@value"/></html:title>
            </html:head>
            <html:body>
             <xsl:call-template name="GlossaryTermData">
              <xsl:with-param name="ParamTerm" select="$VarGlossaryTerm" />
             </xsl:call-template>
            </html:body>
           </html:html>
          </xsl:variable>
          <xsl:variable name="VarResult" select="msxsl:node-set($VarResultAsXML)" />
          <xsl:variable name="VarWriteResult" select="wwexsldoc:Document($VarResult, $VarPath, wwprojext:GetFormatSetting('encoding', 'utf-8'), 'html', '4.0', 'yes')" />
         </xsl:if>

         <wwfiles:File path="{$VarPath}" type="{$ParameterType}" checksum="{wwfilesystem:GetChecksum($VarPath)}" projectchecksum="{$GlobalProject/wwproject:Project/@ChangeID}" groupID="{$VarProjectGroup/@GroupID}" documentID="" actionchecksum="{$GlobalActionChecksum}" category="{$ParameterCategory}" use="{$ParameterUse}" deploy="{$ParameterDeploy}">
          <wwfiles:Depends path="{$VarFilesGlossaryMerge/@path}" checksum="{$VarFilesGlossaryMerge/@checksum}" groupID="{$VarFilesGlossaryMerge/@groupID}" documentID="{$VarFilesGlossaryMerge/@documentID}" />
         </wwfiles:File>

         <xsl:value-of select="wwprogress:End()" />
        </xsl:for-each>
       </xsl:for-each>
      </xsl:for-each>
      <xsl:value-of select="wwprogress:End()" />
     </xsl:for-each>
     <xsl:value-of select="wwprogress:End()" />
    </xsl:for-each>
    <xsl:value-of select="wwprogress:End()" />
   </xsl:if>
  </wwfiles:Files>
 </xsl:template>

 <xsl:template name="GlossaryTermData">
  <xsl:param name="ParamTerm" />

  <xsl:for-each select="$ParamTerm/wwdoc:Paragraph">
   <xsl:variable name="VarParagraph" select="." />
   <html:p>
    <xsl:for-each select="$VarParagraph/wwdoc:TextRun/wwdoc:Text">
     <xsl:value-of select="@value" />
    </xsl:for-each>
   </html:p>
  </xsl:for-each>
 </xsl:template>
</xsl:stylesheet>
