<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns="http://java.sun.com/products/javahelp/index_2_0.dtd"
                              xmlns:javahelpix="http://java.sun.com/products/javahelp/index_2_0.dtd"
                              xmlns:wwsplits="urn:WebWorks-Engine-Splits-Schema"
                              xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                              xmlns:msxsl="urn:schemas-microsoft-com:xslt"
                              xmlns:wwtoc="urn:WebWorks-Engine-TOC-Schema"
                              xmlns:wwlinks="urn:WebWorks-Engine-Links-Schema"
                              xmlns:wwmode="urn:WebWorks-Engine-Mode"
                              xmlns:wwfiles="urn:WebWorks-Engine-Files-Schema"
                              xmlns:wwdoc="urn:WebWorks-Document-Schema"
                              xmlns:wwbehaviors="urn:WebWorks-Behaviors-Schema"
                              xmlns:wwproject="urn:WebWorks-Publish-Project"
                              xmlns:wwpage="urn:WebWorks-Page-Template-Schema"
                              xmlns:wwlocale="urn:WebWorks-Locale-Schema"
                              xmlns:wwprogress="urn:WebWorks-XSLT-Extension-Progress"
                              xmlns:wwlog="urn:WebWorks-XSLT-Extension-Log"
                              xmlns:wwfilesystem="urn:WebWorks-XSLT-Extension-FileSystem"
                              xmlns:wwuri="urn:WebWorks-XSLT-Extension-URI"
                              xmlns:wwstring="urn:WebWorks-XSLT-Extension-StringUtilities"
                              xmlns:wwfilesext="urn:WebWorks-XSLT-Extension-Files"
                              xmlns:wwprojext="urn:WebWorks-XSLT-Extension-Project"
                              xmlns:wwexsldoc="urn:WebWorks-XSLT-Extension-Document"
                              exclude-result-prefixes="wwsplits xsl msxsl wwtoc wwlinks wwmode wwfiles wwdoc wwbehaviors wwsplits wwproject wwpage wwlocale wwprogress wwlog wwfilesystem wwuri wwstring wwfilesext wwprojext wwexsldoc"
>
 <xsl:param name="GlobalInput" />
 <xsl:param name="GlobalPipelineName" />
 <xsl:param name="GlobalProject" />
 <xsl:param name="GlobalFiles" />
 <xsl:param name="ParameterDependsType" />
 <xsl:param name="ParameterSplitsType" />
 <xsl:param name="ParameterType" />
 <xsl:param name="ParameterSplitFileType" />
 <xsl:param name="ParameterCategory" />
 <xsl:param name="ParameterUse" />
 <xsl:param name="ParameterDeploy" />


 <xsl:output encoding="UTF-8" indent="yes" />
 <xsl:namespace-alias stylesheet-prefix="javahelpix" result-prefix="#default" />
 <xsl:strip-space elements="*" />

 <xsl:key name="wwfiles-files-by-type" match="wwfiles:File" use="@type" />
 <xsl:key name="wwfiles-files-by-groupid-type" match="wwfiles:File" use="concat(@groupID, ':', @type)" />
 <xsl:key name="wwsplits-files-by-groupid-type" match="wwsplits:File" use="concat(@groupID, ':', @type)" />

 <xsl:variable name="GlobalActionChecksum">
  <xsl:variable name="VarTransformChecksums">
   <xsl:value-of select="concat(wwuri:AsFilePath('wwtransform:self'), ':', wwfilesystem:GetChecksum(wwuri:AsFilePath('wwtransform:self')))" />
  </xsl:variable>
  <xsl:value-of select="wwstring:MD5Checksum($VarTransformChecksums)" />
 </xsl:variable>

 <xsl:template match="/">
  <wwfiles:Files version="1.0">

   <xsl:if test="wwprojext:GetFormatSetting('enable-glossary-tab', 'false') = 'true'">

    <xsl:variable name="VarProjectGroups" select="$GlobalProject/wwproject:Project/wwproject:Groups/wwproject:Group" />
    <xsl:value-of select="wwprogress:Start(count($VarProjectGroups))" />
    <xsl:for-each select="$VarProjectGroups">
     <xsl:variable name="VarProjectGroup" select="." />
     <xsl:variable name="VarGroupDocuments" select="$VarProjectGroup//wwproject:Document" />

     <xsl:value-of select="wwprogress:Start(1)" />

     <!-- Access behaviors, splits -->
     <!--                          -->
     <xsl:for-each select="$GlobalFiles[1]">
      <xsl:value-of select="wwprogress:Start(1)" />
      <xsl:variable name="VarFilesGlossaryOutput" select="key('wwfiles-files-by-groupid-type', concat($VarProjectGroup/@GroupID, ':', $ParameterDependsType))" />

      <xsl:variable name="VarFilesSplits" select="key('wwfiles-files-by-groupid-type', concat($VarProjectGroup/@GroupID, ':', $ParameterSplitsType))" />
      <xsl:for-each select="$VarFilesSplits[1]">
       <xsl:variable name="VarFilesSplit" select="." />

       <xsl:variable name="VarSplits" select="wwexsldoc:LoadXMLWithoutResolver($VarFilesSplit/@path)" />
       <xsl:for-each select="$VarSplits[1]">
        <xsl:value-of select="wwprogress:Start(1)" />

        <xsl:variable name="VarPath" select="key('wwsplits-files-by-groupid-type', concat($VarProjectGroup/@GroupID, ':', $ParameterSplitFileType))/@path" />

        <!-- Call template -->
        <!--               -->
        <xsl:variable name="VarUpToDate" select="wwfilesext:UpToDate($VarPath, concat($GlobalProject/wwproject:Project/@ChangeID, ':', count($VarFilesGlossaryOutput)), $VarProjectGroup/@GroupID, '', $GlobalActionChecksum)" />
        <xsl:if test="not($VarUpToDate)">
         <xsl:variable name="VarResultAsXML">
          <javahelpix:index version="2.0">
           <xsl:call-template name="GlossaryMap">
            <xsl:with-param name="ParamGlossaryFiles" select="$VarFilesGlossaryOutput" />
            <xsl:with-param name="ParamMapPath" select="$VarPath" />
           </xsl:call-template>
          </javahelpix:index>
         </xsl:variable>
         <xsl:variable name="VarResult" select="msxsl:node-set($VarResultAsXML)" />
         <xsl:variable name="VarWriteResult" select="wwexsldoc:Document($VarResult, $VarPath, wwprojext:GetFormatSetting('encoding', 'utf-8'), 'xml', '1.0', 'yes', 'no', '', '-//Sun Microsystems Inc.//DTD JavaHelp Index Version 2.0//EN', 'http://java.sun.com/products/javahelp/index_2_0.dtd', '', 'text/xml')" />
        </xsl:if>

        <!-- Record files and dependencies -->
        <!--                               -->
        <wwfiles:File path="{$VarPath}" type="{$ParameterType}" checksum="{wwfilesystem:GetChecksum($VarPath)}" projectchecksum="{concat($GlobalProject/wwproject:Project/@ChangeID, ':', count($VarFilesGlossaryOutput))}" groupID="{$VarProjectGroup/@GroupID}" documentID="" actionchecksum="{$GlobalActionChecksum}" category="{$ParameterCategory}" use="{$ParameterUse}" deploy="{$ParameterDeploy}">
         <xsl:for-each select="$VarFilesGlossaryOutput">
          <wwfiles:Depends path="{@path}" checksum="{@checksum}" groupID="{@groupID}" documentID="" />
         </xsl:for-each>
         <wwfiles:Depends path="{$VarFilesSplit/@path}" checksum="{$VarFilesSplit/@checksum}" groupID="{$VarFilesSplit/@groupID}" documentID="" />
        </wwfiles:File>

        <xsl:value-of select="wwprogress:End()" />
       </xsl:for-each>
      </xsl:for-each>
      <xsl:value-of select="wwprogress:End()" />
     </xsl:for-each>

     <xsl:value-of select="wwprogress:End()" />
    </xsl:for-each>
    <xsl:value-of select="wwprogress:End()" />

   </xsl:if>
  </wwfiles:Files>
 </xsl:template>

 <xsl:template name="GlossaryMap">
  <xsl:param name="ParamGlossaryFiles" />
  <xsl:param name="ParamMapPath" />

  <xsl:for-each select="$ParamGlossaryFiles">
   <xsl:variable name="VarFilesGlossaryEntry" select="." />
   <xsl:variable name="VarTermText" select="wwstring:ReplaceWithExpression(wwfilesystem:GetFileName($VarFilesGlossaryEntry/@path), '\.html$', '')" />

   <javahelpix:indexitem text="{$VarTermText}" target="{concat('glossary_', $VarTermText)}" />
  </xsl:for-each>
 </xsl:template>
</xsl:stylesheet>
