<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:javahelpix="http://java.sun.com/products/javahelp/index_2_0.dtd"
                              xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                              xmlns:msxsl="urn:schemas-microsoft-com:xslt"
                              xmlns:wwindex="urn:WebWorks-Index-Schema"
                              xmlns:wwlinks="urn:WebWorks-Engine-Links-Schema"
                              xmlns:wwmode="urn:WebWorks-Engine-Mode"
                              xmlns:wwfiles="urn:WebWorks-Engine-Files-Schema"
                              xmlns:wwdoc="urn:WebWorks-Document-Schema"
                              xmlns:wwsplits="urn:WebWorks-Engine-Splits-Schema"
                              xmlns:wwproject="urn:WebWorks-Publish-Project"
                              xmlns:wwpage="urn:WebWorks-Page-Template-Schema"
                              xmlns:wwlocale="urn:WebWorks-Locale-Schema"
                              xmlns:wwprogress="urn:WebWorks-XSLT-Extension-Progress"
                              xmlns:wwlog="urn:WebWorks-XSLT-Extension-Log"
                              xmlns:wwfilesystem="urn:WebWorks-XSLT-Extension-FileSystem"
                              xmlns:wwuri="urn:WebWorks-XSLT-Extension-URI"
                              xmlns:wwstring="urn:WebWorks-XSLT-Extension-StringUtilities"
                              xmlns:wwfilesext="urn:WebWorks-XSLT-Extension-Files"
                              xmlns:wwprojext="urn:WebWorks-XSLT-Extension-Project"
                              xmlns:wwexsldoc="urn:WebWorks-XSLT-Extension-Document"
                              exclude-result-prefixes="xsl msxsl wwindex wwlinks wwmode wwfiles wwdoc wwsplits wwproject wwpage wwlocale wwprogress wwlog wwfilesystem wwuri wwstring wwfilesext wwprojext wwexsldoc"
>
 <xsl:param name="GlobalInput" />
 <xsl:param name="GlobalPipelineName" />
 <xsl:param name="GlobalProject" />
 <xsl:param name="GlobalFiles" />
 <xsl:param name="ParameterDependsType" />
 <xsl:param name="ParameterSplitsType" />
 <xsl:param name="ParameterType" />
 <xsl:param name="ParameterLocaleType" />
 <xsl:param name="ParameterTOCSplitFileType" />
 <xsl:param name="ParameterIndexSplitFileType" />
 <xsl:param name="ParameterCategory" />
 <xsl:param name="ParameterUse" />
 <xsl:param name="ParameterDeploy" />


 <xsl:output encoding="UTF-8" indent="yes" />
 <xsl:namespace-alias stylesheet-prefix="javahelpix" result-prefix="#default" />
 <xsl:strip-space elements="*" />


 <xsl:key name="wwfiles-files-by-type" match="wwfiles:File" use="@type" />
 <xsl:key name="wwfiles-files-by-groupid-type" match="wwfiles:File" use="concat(@groupID, ':', @type)" />
 <xsl:key name="wwsplits-files-by-groupid-type" match="wwsplits:File" use="concat(@groupID, ':', @type)" />


 <xsl:variable name="GlobalActionChecksum">
  <xsl:variable name="VarTransformChecksums">
   <xsl:value-of select="concat(wwuri:AsFilePath('wwtransform:self'), ':', wwfilesystem:GetChecksum(wwuri:AsFilePath('wwtransform:self')))" />
  </xsl:variable>
  <xsl:value-of select="wwstring:MD5Checksum($VarTransformChecksums)" />
 </xsl:variable>


 <!-- Locale -->
 <!--        -->
 <xsl:variable name="GlobalLocalePath">
  <xsl:for-each select="$GlobalFiles[1]">
   <xsl:value-of select="key('wwfiles-files-by-type', $ParameterLocaleType)[1]/@path" />
  </xsl:for-each>
 </xsl:variable>
 <xsl:variable name="GlobalLocale" select="wwexsldoc:LoadXMLWithoutResolver($GlobalLocalePath)" />


 <xsl:template match="/">
  <wwfiles:Files version="1.0">

   <!-- Generate Index? -->
   <!--                 -->
   <xsl:if test="wwprojext:GetFormatSetting('index-generate', 'true') = 'true'">
    <!-- Iterate input documents -->
    <!--                         -->
    <xsl:for-each select="$GlobalFiles[1]">
     <xsl:variable name="VarFilesByType" select="key('wwfiles-files-by-type', $ParameterDependsType)" />

     <xsl:value-of select="wwprogress:Start(count($VarFilesByType))" />

     <xsl:for-each select="$VarFilesByType">
      <xsl:value-of select="wwprogress:Start(1)" />

      <xsl:variable name="VarFilesDocument" select="." />

      <!-- Load splits -->
      <!--             -->
      <xsl:variable name="VarFilesSplits" select="key('wwfiles-files-by-groupid-type', concat($VarFilesDocument/@groupID, ':', $ParameterSplitsType))[1]" />
      <xsl:variable name="VarSplits" select="wwexsldoc:LoadXMLWithoutResolver($VarFilesSplits/@path)" />

      <xsl:for-each select="$VarSplits[1]">
       <!-- Split -->
       <!--       -->
       <xsl:variable name="VarSplit" select="key('wwsplits-files-by-groupid-type', concat($VarFilesDocument/@groupID, ':', $ParameterIndexSplitFileType))[1]" />

       <!-- Transform -->
       <!--           -->
       <xsl:variable name="VarUpToDate" select="wwfilesext:UpToDate($VarSplit/@path, '', $VarFilesDocument/@groupID, $VarFilesDocument/@documentID, $GlobalActionChecksum)" />
       <xsl:if test="not($VarUpToDate)">
        <xsl:variable name="VarResultAsXML">
         <!-- Load document -->
         <!--               -->
         <xsl:variable name="VarIndex" select="wwexsldoc:LoadXMLWithoutResolver($VarFilesDocument/@path)" />

         <xsl:call-template name="Index">
          <xsl:with-param name="ParamSplits" select="$VarSplits" />
          <xsl:with-param name="ParamIndex" select="$VarIndex" />
          <xsl:with-param name="ParamSplit" select="$VarSplit" />
         </xsl:call-template>
        </xsl:variable>
        <xsl:variable name="VarResult" select="msxsl:node-set($VarResultAsXML)" />
        <xsl:variable name="VarWriteResult" select="wwexsldoc:Document($VarResult, $VarSplit/@path, wwprojext:GetFormatSetting('encoding', 'utf-8'), 'xml', '1.0', 'yes', 'no', '', '-//Sun Microsystems Inc.//DTD JavaHelp Index Version 2.0//EN', 'http://java.sun.com/products/javahelp/index_2_0.dtd', '', 'text/xml')" />
       </xsl:if>

       <!-- Report Files -->
       <!--              -->
       <wwfiles:File path="{$VarSplit/@path}" type="{$ParameterType}" checksum="{wwfilesystem:GetChecksum($VarSplit/@path)}" projectchecksum="" groupID="{$VarFilesDocument/@groupID}" documentID="{$VarFilesDocument/@documentID}" actionchecksum="{$GlobalActionChecksum}" category="{$ParameterCategory}" use="{$ParameterUse}" deploy="{$ParameterDeploy}">
        <wwfiles:Depends path="{$GlobalLocalePath}" checksum="{wwfilesystem:GetChecksum($GlobalLocalePath)}" groupID="" documentID="" />
        <wwfiles:Depends path="{$VarFilesSplits/@path}" checksum="{$VarFilesSplits/@checksum}" groupID="{$VarFilesSplits/@groupID}" documentID="{$VarFilesSplits/@documentID}" />
        <wwfiles:Depends path="{$VarFilesDocument/@path}" checksum="{$VarFilesDocument/@checksum}" groupID="{$VarFilesDocument/@groupID}" documentID="{$VarFilesDocument/@documentID}" />
       </wwfiles:File>
      </xsl:for-each>

      <xsl:value-of select="wwprogress:End()" />
     </xsl:for-each>

     <xsl:value-of select="wwprogress:End()" />
    </xsl:for-each>
   </xsl:if>

  </wwfiles:Files>
 </xsl:template>


 <xsl:template name="Index">
  <xsl:param name="ParamSplits" />
  <xsl:param name="ParamIndex" />
  <xsl:param name="ParamSplit" />

  <javahelpix:index version="2.0">
   <xsl:call-template name="Sections">
    <xsl:with-param name="ParamSplits" select="$ParamSplits" />
    <xsl:with-param name="ParamSplit" select="$ParamSplit" />
    <xsl:with-param name="ParamIndex" select="$ParamIndex/wwindex:Index" />
   </xsl:call-template>
  </javahelpix:index>
 </xsl:template>


 <xsl:template name="Sections">
  <xsl:param name="ParamSplits" />
  <xsl:param name="ParamSplit" />
  <xsl:param name="ParamIndex" />

  <xsl:for-each select="$ParamIndex/wwindex:Section">
   <xsl:variable name="VarSection" select="." />

    <xsl:call-template name="Groups">
     <xsl:with-param name="ParamSplits" select="$ParamSplits" />
     <xsl:with-param name="ParamSplit" select="$ParamSplit" />
     <xsl:with-param name="ParamSection" select="$VarSection" />
    </xsl:call-template>

  </xsl:for-each>
 </xsl:template>


 <xsl:template name="Groups">
  <xsl:param name="ParamSplits" />
  <xsl:param name="ParamSplit" />
  <xsl:param name="ParamSection" />

  <xsl:for-each select="$ParamSection/wwindex:Group">
   <xsl:variable name="VarGroup" select="." />

   <xsl:comment><xsl:value-of select="$VarGroup/@name" /></xsl:comment>

     <xsl:call-template name="Entries">
      <xsl:with-param name="ParamSplits" select="$ParamSplits" />
      <xsl:with-param name="ParamSplit" select="$ParamSplit" />
      <xsl:with-param name="ParamParent" select="$VarGroup" />
     </xsl:call-template>

  </xsl:for-each>
 </xsl:template>


 <xsl:template name="Entries">
  <xsl:param name="ParamSplits" />
  <xsl:param name="ParamSplit" />
  <xsl:param name="ParamParent" />

  <xsl:for-each select="$ParamParent/wwindex:Entry">
   <javahelpix:indexitem>
   <xsl:variable name="VarEntry" select="." />

    <xsl:variable name="VarSee" select="$VarEntry/wwindex:See[1]" />

    <xsl:choose>
     <xsl:when test="count($VarSee) = 1">

      <!-- See/See Also -->
      <!--              -->

       <xsl:attribute name="text">
        <xsl:call-template name="Content">
         <xsl:with-param name="ParamContent" select="$VarEntry/wwdoc:Content[1]" />
        </xsl:call-template>
       </xsl:attribute>

     </xsl:when>

     <xsl:otherwise>
      <!-- Regular entry -->
      <!--               -->
      <xsl:variable name="VarLinks" select="$VarEntry/wwindex:Link" />

      <xsl:choose>
       <xsl:when test="count($VarLinks) = 0">
        <!-- Emit entry without any links -->
        <!--                              -->
        <xsl:attribute name="text">
         <xsl:call-template name="Content">
          <xsl:with-param name="ParamContent" select="$VarEntry/wwdoc:Content[1]" />
         </xsl:call-template>
        </xsl:attribute>
       </xsl:when>

       <xsl:when test="(count($VarLinks) = 1) and (count($VarLinks[1]/wwdoc:Content[1]) = 0)">
        <!-- Wrap link directly around entry -->
        <!--                                 -->
        <xsl:variable name="VarLink" select="$VarLinks[1]" />
        <xsl:variable name="VarLinkDocumentID" select="$ParamSplits/wwsplits:Splits/wwsplits:Split[@path = $VarLink/@href]/@documentID" />

        <xsl:attribute name="target">
         <xsl:value-of select="concat($VarLinkDocumentID, '_', $VarLink/@anchor)" />
        </xsl:attribute>

        <xsl:attribute name="text">
         <xsl:call-template name="Content">
          <xsl:with-param name="ParamContent" select="$VarEntry/wwdoc:Content[1]" />
         </xsl:call-template>
        </xsl:attribute>

       </xsl:when>

       <xsl:otherwise>
        <!-- Emit entry the text of the parent followed by elements  -->
        <!-- for each child link.                                    -->

        <xsl:attribute name="text">
         <xsl:call-template name="Content">
          <xsl:with-param name="ParamContent" select="$VarEntry/wwdoc:Content[1]" />
         </xsl:call-template>
        </xsl:attribute>

         <!-- Links -->
         <!--       -->
         <xsl:for-each select="$VarLinks">
          <javahelpix:indexitem>
           <xsl:variable name="VarLink" select="." />

           <xsl:variable name="VarEntryLinkDocumentID" select="$ParamSplits//wwsplits:Split[@path = $VarLink/@href]/@documentID" />

          <xsl:choose>
           <xsl:when test="count($VarLink/wwdoc:Content[1]) = 1">
            <!-- Link has explicit text to show -->
            <!--                                -->
            <xsl:variable name="VarLinkSee" select="$VarLink/wwdoc:See[1]" />
            <xsl:choose>
             <xsl:when test="count($VarLinkSee) = 1">
              <!-- See/See Also link redirect -->
              <!--                            -->
              <xsl:attribute name="text">
               <xsl:call-template name="Content">
                <xsl:with-param name="ParamContent" select="$VarLink/wwdoc:Content[1]" />
               </xsl:call-template>
              </xsl:attribute>

             </xsl:when>

             <xsl:otherwise>
              <!-- Wrap link around content -->
              <!--                          -->
              <xsl:attribute name="target">
               <xsl:value-of select="concat($VarEntryLinkDocumentID, '_', $VarLink/@anchor)"/>
              </xsl:attribute>

              <xsl:attribute name="text">
               <xsl:call-template name="Content">
                <xsl:with-param name="ParamContent" select="$VarLink/wwdoc:Content[1]" />
               </xsl:call-template>
              </xsl:attribute>

             </xsl:otherwise>
            </xsl:choose>
           </xsl:when>

           <xsl:otherwise>

            <xsl:attribute name="target">
             <xsl:value-of select="concat($VarEntryLinkDocumentID, '_', $VarLink/@anchor)"/>
            </xsl:attribute>

            <xsl:attribute name="text">

             <!-- Emit numbered text -->
             <!--                    -->
             <xsl:value-of select="concat('(', position(), ')')" />

             <xsl:call-template name="Content">
              <xsl:with-param name="ParamContent" select="$VarEntry/wwdoc:Content[1]" />
             </xsl:call-template>
            </xsl:attribute>

           </xsl:otherwise>

          </xsl:choose>
          </javahelpix:indexitem>
         </xsl:for-each>

       </xsl:otherwise>
      </xsl:choose>

     </xsl:otherwise>
    </xsl:choose>

    <!-- Process children -->
    <!--                  -->
    <xsl:for-each select="$VarEntry/wwindex:Entry[1]">

      <xsl:call-template name="Entries">
       <xsl:with-param name="ParamSplits" select="$ParamSplits" />
       <xsl:with-param name="ParamSplit" select="$ParamSplit" />
       <xsl:with-param name="ParamParent" select="$VarEntry" />
      </xsl:call-template>

     </xsl:for-each>
   </javahelpix:indexitem>
  </xsl:for-each>

 </xsl:template>


 <xsl:template name="Content">
  <xsl:param name="ParamContent" />

  <!-- Simple processing for now -->
  <!--                           -->
  <xsl:value-of select="$ParamContent/wwdoc:TextRun/wwdoc:Text/@value" />
 </xsl:template>
</xsl:stylesheet>
