<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns="http://java.sun.com/products/javahelp/helpset_2_0.dtd"
                              xmlns:javahelphs="http://java.sun.com/products/javahelp/helpset_2_0.dtd"
                              xmlns:javahelpmap="http://java.sun.com/products/javahelp/map_2_0.dtd"
                              xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                              xmlns:msxsl="urn:schemas-microsoft-com:xslt"
                              xmlns:wwmode="urn:WebWorks-Engine-Mode"
                              xmlns:wwfiles="urn:WebWorks-Engine-Files-Schema"
                              xmlns:wwdoc="urn:WebWorks-Document-Schema"
                              xmlns:wwsplits="urn:WebWorks-Engine-Splits-Schema"
                              xmlns:wwproject="urn:WebWorks-Publish-Project"
                              xmlns:wwlocale="urn:WebWorks-Locale-Schema"
                              xmlns:wwprogress="urn:WebWorks-XSLT-Extension-Progress"
                              xmlns:wwlog="urn:WebWorks-XSLT-Extension-Log"
                              xmlns:wwfilesystem="urn:WebWorks-XSLT-Extension-FileSystem"
                              xmlns:wwuri="urn:WebWorks-XSLT-Extension-URI"
                              xmlns:wwstring="urn:WebWorks-XSLT-Extension-StringUtilities"
                              xmlns:wwunits="urn:WebWorks-XSLT-Extension-Units"
                              xmlns:wwfilesext="urn:WebWorks-XSLT-Extension-Files"
                              xmlns:wwprojext="urn:WebWorks-XSLT-Extension-Project"
                              xmlns:wwexsldoc="urn:WebWorks-XSLT-Extension-Document"
                              exclude-result-prefixes="javahelpmap xsl msxsl wwmode wwfiles wwdoc wwsplits wwproject wwlocale wwprogress wwlog wwfilesystem wwuri wwstring wwunits wwfilesext wwprojext wwexsldoc"
>
 <xsl:param name="GlobalInput" />
 <xsl:param name="GlobalPipelineName" />
 <xsl:param name="GlobalProject" />
 <xsl:param name="GlobalFiles" />
 <xsl:param name="ParameterDependsType" />
 <xsl:param name="ParameterSplitsType" />
 <xsl:param name="ParameterLocaleType" />
 <xsl:param name="ParameterTOCType" />
 <xsl:param name="ParameterIndexType" />
 <xsl:param name="ParameterGlossaryType" />
 <xsl:param name="ParameterSearchDataDirectoryFileType" />
 <xsl:param name="ParameterType" />
 <xsl:param name="ParameterMergeType" />
 <xsl:param name="ParameterSplitFileType" />
 <xsl:param name="ParameterCategory" />
 <xsl:param name="ParameterUse" />
 <xsl:param name="ParameterDeploy" />


 <xsl:output method="xml" encoding="UTF-8" indent="yes" />
 <xsl:namespace-alias stylesheet-prefix="javahelphs" result-prefix="#default" />
 <xsl:strip-space elements="*" />

 <xsl:key name="wwfiles-files-by-type" match="wwfiles:File" use="@type" />
 <xsl:key name="wwfiles-files-by-groupid-type" match="wwfiles:File" use="concat(@groupID, ':', @type)" />
 <xsl:key name="wwsplits-files-by-groupid-type" match="wwsplits:File" use="concat(@groupID, ':', @type)" />
 <xsl:key name="wwproject-formatconfigurations-by-targetid" match="wwproject:FormatConfiguration" use="@TargetID" />


 <xsl:variable name="GlobalActionChecksum">
  <xsl:variable name="VarTransformChecksums">
   <xsl:value-of select="concat(wwuri:AsFilePath('wwtransform:self'), ':', wwfilesystem:GetChecksum(wwuri:AsFilePath('wwtransform:self')))" />
  </xsl:variable>
  <xsl:value-of select="wwstring:MD5Checksum($VarTransformChecksums)" />
 </xsl:variable>

 <!-- Locale -->
 <!--        -->
 <xsl:variable name="GlobalLocalePath">
  <xsl:for-each select="$GlobalFiles[1]">
   <xsl:value-of select="key('wwfiles-files-by-type', $ParameterLocaleType)[1]/@path" />
  </xsl:for-each>
 </xsl:variable>
 <xsl:variable name="GlobalLocale" select="wwexsldoc:LoadXMLWithoutResolver($GlobalLocalePath)" />


 <xsl:template match="/">
  <wwfiles:Files version="1.0">

   <!-- Groups -->
   <!--        -->
   <xsl:variable name="VarProjectGroups" select="$GlobalProject/wwproject:Project/wwproject:Groups/wwproject:Group" />
   <xsl:value-of select="wwprogress:Start(count($VarProjectGroups) + 1)" />
   <xsl:for-each select="$VarProjectGroups">
    <xsl:variable name="VarProjectGroup" select="." />
    <xsl:variable name="VarGroupDocuments" select="$VarProjectGroup/wwproject:Document" />

    <xsl:value-of select="wwprogress:Start(1)" />

    <!-- Access splits -->
    <!--               -->
    <xsl:for-each select="$GlobalFiles[1]">
     <xsl:variable name="VarFilesNameInfo" select="key('wwfiles-files-by-groupid-type', concat($VarProjectGroup/@GroupID, ':', $ParameterSplitsType))" />
     <xsl:value-of select="wwprogress:Start(1)" />

     <xsl:for-each select="$VarFilesNameInfo[1]">
      <xsl:variable name="VarSplitsFileInfo" select="." />

      <xsl:value-of select="wwprogress:Start(1)" />

      <!-- Load splits -->
      <!--             -->
      <xsl:variable name="VarSplits" select="wwexsldoc:LoadXMLWithoutResolver($VarSplitsFileInfo/@path)" />

      <!-- Iterate input documents -->
      <!--                         -->
      <xsl:for-each select="$GlobalFiles[1]">
       <xsl:variable name="VarFilesByType" select="key('wwfiles-files-by-groupid-type', concat($VarProjectGroup/@GroupID, ':', $ParameterDependsType))" />
       <xsl:variable name="VarTOCByType" select="key('wwfiles-files-by-groupid-type', concat($VarProjectGroup/@GroupID, ':', $ParameterTOCType))" />
       <xsl:variable name="VarIndexByType" select="key('wwfiles-files-by-groupid-type', concat($VarProjectGroup/@GroupID, ':', $ParameterIndexType))" />
       <xsl:variable name="VarGlossaryByType" select="key('wwfiles-files-by-groupid-type', concat($VarProjectGroup/@GroupID, ':', $ParameterGlossaryType))" />

       <xsl:value-of select="wwprogress:Start(count($VarFilesByType))" />

       <xsl:for-each select="$VarFilesByType[1]">
        <xsl:value-of select="wwprogress:Start(1)" />

        <xsl:variable name="VarFilesMapNode" select="." />
        <xsl:variable name="VarMapPath" select="$VarFilesMapNode/@path" />
        <xsl:variable name="VarMapFile" select="wwexsldoc:LoadXMLWithoutResolver($VarMapPath)" />
        <xsl:variable name="VarTOCPath" select="$VarTOCByType[1]/@path" />
        <xsl:variable name="VarIndexPath" select="$VarIndexByType[1]/@path" />
        <xsl:variable name="VarGlossaryPath" select="$VarGlossaryByType/@path" />

        <!-- Search data directory path -->
        <!--                            -->
        <xsl:variable name="VarSearchDataDirectoryPath">
         <xsl:for-each select="$VarSplits[1]">
          <xsl:value-of select="key('wwsplits-files-by-groupid-type', concat($VarProjectGroup/@GroupID, ':', $ParameterSearchDataDirectoryFileType))[1]/@path" />
         </xsl:for-each>
        </xsl:variable>

        <!-- Output Path -->
        <!--             -->
        <xsl:variable name="VarPath">
         <xsl:for-each select="$VarSplits[1]">
          <xsl:value-of select="key('wwsplits-files-by-groupid-type', concat($VarProjectGroup/@GroupID, ':', $ParameterSplitFileType))[1]/@path" />
         </xsl:for-each>
        </xsl:variable>

        <!-- Call template -->
        <!--               -->
        <xsl:variable name="VarUpToDate" select="wwfilesext:UpToDate($VarPath, $GlobalProject/wwproject:Project/@ChangeID, $VarFilesMapNode/@groupID, '', $GlobalActionChecksum)" />
        <xsl:if test="not($VarUpToDate)">
         <xsl:variable name="VarResultAsXML">
          <xsl:call-template name="Helpset">
           <xsl:with-param name="ParamPath" select="$VarPath" />
           <xsl:with-param name="ParamMapPath" select="$VarMapPath" />
           <xsl:with-param name="ParamMapFile" select="$VarMapFile" />
           <xsl:with-param name="ParamTOCPath" select="$VarTOCPath" />
           <xsl:with-param name="ParamIndexPath" select="$VarIndexPath" />
           <xsl:with-param name="ParamGlossaryPath" select="$VarGlossaryPath" />
           <xsl:with-param name="ParamSearchDataDirectoryPath" select="$VarSearchDataDirectoryPath" />
          </xsl:call-template>
         </xsl:variable>
         <xsl:variable name="VarResult" select="msxsl:node-set($VarResultAsXML)" />
         <xsl:variable name="VarWriteResult" select="wwexsldoc:Document($VarResult, $VarPath, wwprojext:GetFormatSetting('encoding', 'utf-8'), 'xml', '1.0', 'yes', 'no', '', '-//Sun Microsystems Inc.//DTD JavaHelp HelpSet Version 2.0//EN', 'http://java.sun.com/products/javahelp/helpset_2_0.dtd', '', 'text/xml')" />
        </xsl:if>

        <wwfiles:File path="{$VarPath}" type="{$ParameterType}" checksum="{wwfilesystem:GetChecksum($VarPath)}" projectchecksum="{$GlobalProject/wwproject:Project/@ChangeID}" groupID="{$VarFilesMapNode/@groupID}" documentID="" actionchecksum="{$GlobalActionChecksum}" category="{$ParameterCategory}" use="{$ParameterUse}" deploy="{$ParameterDeploy}">
         <wwfiles:Depends path="{$GlobalLocalePath}" checksum="{wwfilesystem:GetChecksum($GlobalLocalePath)}" groupID="" documentID="" />
         <wwfiles:Depends path="{$VarFilesMapNode/@path}" checksum="{$VarFilesMapNode/@checksum}" groupID="{$VarFilesMapNode/@groupID}" documentID="{$VarFilesMapNode/@documentID}" />
        </wwfiles:File>


        <xsl:value-of select="wwprogress:End()" />
       </xsl:for-each>

       <xsl:value-of select="wwprogress:End()" />
      </xsl:for-each>

      <xsl:value-of select="wwprogress:End()" />
     </xsl:for-each>

     <xsl:value-of select="wwprogress:End()" />
    </xsl:for-each>

    <xsl:value-of select="wwprogress:End()" />
   </xsl:for-each>

   <!-- Merge -->
   <!--       -->
   <xsl:value-of select="wwprogress:Start(1)" />
   <xsl:if test="count($VarProjectGroups) &gt; 1">
    <xsl:for-each select="$GlobalProject[1]">
     <xsl:variable name="VarFormatConfiguration" select="key('wwproject-formatconfigurations-by-targetid', wwprojext:GetFormatID())[1]" />
     <xsl:variable name="VarMergeSettings" select="$VarFormatConfiguration/wwproject:MergeSettings" />

     <xsl:variable name="VarMergePath" select="wwfilesystem:Combine(wwprojext:GetTargetOutputDirectoryPath(), concat(wwprojext:GetFormatName(), '.hs'))" />

     <!-- Call template -->
     <!--               -->
     <xsl:variable name="VarUpToDate" select="wwfilesext:UpToDate($VarMergePath, concat($GlobalProject/wwproject:Project/@ChangeID, ':', count($VarProjectGroups)), '', '', $GlobalActionChecksum)" />
     <xsl:if test="not($VarUpToDate)">
      <xsl:variable name="VarResultAsXML">
       <xsl:call-template name="MergeHelpset">
        <xsl:with-param name="ParamMergePath" select="$VarMergePath" />
        <xsl:with-param name="ParamMergeSettings" select="$VarMergeSettings" />
       </xsl:call-template>
      </xsl:variable>
      <xsl:variable name="VarResult" select="msxsl:node-set($VarResultAsXML)" />
      <xsl:variable name="VarWriteResult" select="wwexsldoc:Document($VarResult, $VarMergePath, wwprojext:GetFormatSetting('encoding', 'utf-8'), 'xml', '1.0', 'yes')" />
     </xsl:if>

     <wwfiles:File path="{$VarMergePath}" type="{$ParameterMergeType}" checksum="{wwfilesystem:GetChecksum($VarMergePath)}" projectchecksum="{concat($GlobalProject/wwproject:Project/@ChangeID, ':', count($VarProjectGroups))}" groupID="" documentID="" actionchecksum="{$GlobalActionChecksum}" category="{$ParameterCategory}" use="{$ParameterUse}" deploy="{$ParameterDeploy}">
      <wwfiles:Depends path="{$GlobalLocalePath}" checksum="{wwfilesystem:GetChecksum($GlobalLocalePath)}" groupID="" documentID="" />
     </wwfiles:File>
    </xsl:for-each>
   </xsl:if>
   <xsl:value-of select="wwprogress:End()" />

   <xsl:value-of select="wwprogress:End()" />

  </wwfiles:Files>
 </xsl:template>


 <xsl:template name="Helpset">
  <xsl:param name="ParamPath" />
  <xsl:param name="ParamMapPath" />
  <xsl:param name="ParamMapFile" />
  <xsl:param name="ParamTOCPath" />
  <xsl:param name="ParamIndexPath" />
  <xsl:param name="ParamGlossaryPath" />
  <xsl:param name="ParamSearchDataDirectoryPath" />

  <xsl:variable name="VarTitle" select="wwprojext:GetFormatSetting('helpset-title', wwfilesystem:GetFileNameWithoutExtension($ParamMapPath))" />
  <xsl:variable name="VarHomeID" select="$ParamMapFile/javahelpmap:map/javahelpmap:mapID[1]/@target" />

  <javahelphs:helpset version="2.0">
   <javahelphs:title><xsl:value-of select="$VarTitle" /></javahelphs:title>
   <javahelphs:maps>
    <javahelphs:homeID><xsl:value-of select="$VarHomeID" /></javahelphs:homeID>
    <javahelphs:mapref location="{wwuri:Unescape(wwuri:GetRelativeTo($ParamMapPath, $ParamPath))}" />
   </javahelphs:maps>

   <xsl:if test="wwprojext:GetFormatSetting('toc-generate', 'true') = 'true'">
    <javahelphs:view>
     <javahelphs:name><xsl:value-of select="$GlobalLocale/wwlocale:Locale/wwlocale:Strings/wwlocale:String[@name = 'TOCTabLabel']/@value" /></javahelphs:name>
     <javahelphs:label><xsl:value-of select="'Contents'" /></javahelphs:label>
     <javahelphs:type><xsl:value-of select="'javax.help.TOCView'" /></javahelphs:type>
     <javahelphs:data><xsl:value-of select="wwuri:Unescape(wwuri:GetRelativeTo($ParamTOCPath, $ParamPath))" /></javahelphs:data>
    </javahelphs:view>
   </xsl:if>

   <xsl:if test="wwprojext:GetFormatSetting('index-generate', 'true') = 'true'">
    <javahelphs:view>
     <javahelphs:name><xsl:value-of select="$GlobalLocale/wwlocale:Locale/wwlocale:Strings/wwlocale:String[@name = 'IXTabLabel']/@value" /></javahelphs:name>
     <javahelphs:label><xsl:value-of select="'Index'" /></javahelphs:label>
     <javahelphs:type><xsl:value-of select="'javax.help.IndexView'" /></javahelphs:type>
     <javahelphs:data><xsl:value-of select="wwuri:Unescape(wwuri:GetRelativeTo($ParamIndexPath, $ParamPath))" /></javahelphs:data>
    </javahelphs:view>
   </xsl:if>

   <xsl:if test="wwprojext:GetFormatSetting('enable-favorites-tab', 'true') = 'true'">
    <javahelphs:view>
     <javahelphs:name><xsl:value-of select="$GlobalLocale/wwlocale:Locale/wwlocale:Strings/wwlocale:String[@name = 'FavoritesTabLabel']/@value" /></javahelphs:name>
     <javahelphs:label><xsl:value-of select="'Favorites'" /></javahelphs:label>
     <javahelphs:type><xsl:value-of select="'javax.help.FavoritesView'" /></javahelphs:type>
    </javahelphs:view>
   </xsl:if>

   <xsl:if test="wwprojext:GetFormatSetting('enable-search-tab', 'true') = 'true'">
    <javahelphs:view>
     <javahelphs:name><xsl:value-of select="$GlobalLocale/wwlocale:Locale/wwlocale:Strings/wwlocale:String[@name = 'SearchTabLabel']/@value" /></javahelphs:name>
     <javahelphs:label><xsl:value-of select="'Search'" /></javahelphs:label>
     <javahelphs:type><xsl:value-of select="'javax.help.SearchView'" /></javahelphs:type>
     <javahelphs:data engine="com.sun.java.help.search.DefaultSearchEngine"><xsl:value-of select="wwuri:Unescape(wwuri:GetRelativeTo($ParamSearchDataDirectoryPath, $ParamPath))" /></javahelphs:data>
    </javahelphs:view>
   </xsl:if>

   <xsl:if test="wwprojext:GetFormatSetting('enable-glossary-tab', 'false') = 'true'">
    <javahelphs:view>
     <javahelphs:name><xsl:value-of select="$GlobalLocale/wwlocale:Locale/wwlocale:Strings/wwlocale:String[@name = 'GlossaryTabLabel']/@value" /></javahelphs:name>
     <javahelphs:label><xsl:value-of select="'Glossary'" /></javahelphs:label>
     <javahelphs:type><xsl:value-of select="'javax.help.GlossaryView'" /></javahelphs:type>
     <javahelphs:data><xsl:value-of select="wwuri:Unescape(wwuri:GetRelativeTo($ParamGlossaryPath, $ParamPath))" /></javahelphs:data>
    </javahelphs:view>
   </xsl:if>

  </javahelphs:helpset>
 </xsl:template>


 <xsl:template name="MergeHelpset">
  <xsl:param name="ParamMergePath" />
  <xsl:param name="ParamMergeSettings" />

  <xsl:variable name="VarMergeTitle">
   <xsl:choose>
    <xsl:when test="string-length($ParamMergeSettings/@Title) &gt; 0">
     <xsl:value-of select="$ParamMergeSettings/@Title" />
    </xsl:when>

    <xsl:otherwise>
     <xsl:value-of select="wwprojext:GetFormatName()" />
    </xsl:otherwise>
   </xsl:choose>
  </xsl:variable>

  <javahelphs:helpset version="2.0">
   <javahelphs:title><xsl:value-of select="$VarMergeTitle" /></javahelphs:title>

   <xsl:if test="wwprojext:GetFormatSetting('toc-generate', 'true') = 'true'">
    <javahelphs:view>
     <javahelphs:name><xsl:value-of select="$GlobalLocale/wwlocale:Locale/wwlocale:Strings/wwlocale:String[@name = 'TOCTabLabel']/@value" /></javahelphs:name>
     <javahelphs:label><xsl:value-of select="'Contents'" /></javahelphs:label>
     <javahelphs:type><xsl:value-of select="'javax.help.TOCView'" /></javahelphs:type>
    </javahelphs:view>
   </xsl:if>

   <xsl:if test="wwprojext:GetFormatSetting('index-generate', 'true') = 'true'">
    <javahelphs:view>
     <javahelphs:name><xsl:value-of select="$GlobalLocale/wwlocale:Locale/wwlocale:Strings/wwlocale:String[@name = 'IXTabLabel']/@value" /></javahelphs:name>
     <javahelphs:label><xsl:value-of select="'Index'" /></javahelphs:label>
     <javahelphs:type><xsl:value-of select="'javax.help.IndexView'" /></javahelphs:type>
    </javahelphs:view>
   </xsl:if>

   <xsl:if test="wwprojext:GetFormatSetting('enable-favorites-tab', 'true') = 'true'">
    <javahelphs:view>
     <javahelphs:name><xsl:value-of select="$GlobalLocale/wwlocale:Locale/wwlocale:Strings/wwlocale:String[@name = 'FavoritesTabLabel']/@value" /></javahelphs:name>
     <javahelphs:label><xsl:value-of select="'Favorites'" /></javahelphs:label>
     <javahelphs:type><xsl:value-of select="'javax.help.FavoritesView'" /></javahelphs:type>
    </javahelphs:view>
   </xsl:if>

   <xsl:if test="wwprojext:GetFormatSetting('enable-search-tab', 'true') = 'true'">
    <javahelphs:view>
     <javahelphs:name><xsl:value-of select="$GlobalLocale/wwlocale:Locale/wwlocale:Strings/wwlocale:String[@name = 'SearchTabLabel']/@value" /></javahelphs:name>
     <javahelphs:label><xsl:value-of select="'Search'" /></javahelphs:label>
     <javahelphs:type><xsl:value-of select="'javax.help.SearchView'" /></javahelphs:type>
    </javahelphs:view>
   </xsl:if>

   <xsl:if test="wwprojext:GetFormatSetting('enable-glossary-tab', 'false') = 'true'">
    <javahelphs:view>
     <javahelphs:name><xsl:value-of select="$GlobalLocale/wwlocale:Locale/wwlocale:Strings/wwlocale:String[@name = 'GlossaryTabLabel']/@value" /></javahelphs:name>
     <javahelphs:label><xsl:value-of select="'Glossary'" /></javahelphs:label>
     <javahelphs:type><xsl:value-of select="'javax.help.GlossaryView'" /></javahelphs:type>
    </javahelphs:view>
   </xsl:if>

   <!-- Groups -->
   <!--        -->
   <xsl:for-each select="$ParamMergeSettings//wwproject:MergeGroup">
    <xsl:variable name="VarMergeGroup" select="." />

    <!-- Access splits -->
    <!--               -->
    <xsl:for-each select="$GlobalFiles[1]">
     <xsl:variable name="VarFilesNameInfo" select="key('wwfiles-files-by-groupid-type', concat($VarMergeGroup/@GroupID, ':', $ParameterSplitsType))" />

     <xsl:for-each select="$VarFilesNameInfo[1]">
      <xsl:variable name="VarSplitsFileInfo" select="." />

      <!-- Load splits -->
      <!--             -->
      <xsl:variable name="VarSplits" select="wwexsldoc:LoadXMLWithoutResolver($VarSplitsFileInfo/@path)" />

      <!-- Output Path -->
      <!--             -->
      <xsl:variable name="VarPath">
       <xsl:for-each select="$VarSplits[1]">
        <xsl:value-of select="key('wwsplits-files-by-groupid-type', concat($VarMergeGroup/@GroupID, ':', $ParameterSplitFileType))[1]/@path" />
       </xsl:for-each>
      </xsl:variable>

      <javahelphs:subhelpset location="{wwuri:Unescape(wwuri:GetRelativeTo($VarPath, $ParamMergePath))}" />
     </xsl:for-each>
    </xsl:for-each>
   </xsl:for-each>
  </javahelphs:helpset>
 </xsl:template>
</xsl:stylesheet>
