<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                              xmlns:msxsl="urn:schemas-microsoft-com:xslt"
                              xmlns:wwmode="urn:WebWorks-Engine-Mode"
                              xmlns:wwfiles="urn:WebWorks-Engine-Files-Schema"
                              xmlns:wwdoc="urn:WebWorks-Document-Schema"
                              xmlns:wwsplits="urn:WebWorks-Engine-Splits-Schema"
                              xmlns:wwproject="urn:WebWorks-Publish-Project"
                              xmlns:wwprogress="urn:WebWorks-XSLT-Extension-Progress"
                              xmlns:wwlog="urn:WebWorks-XSLT-Extension-Log"
                              xmlns:wwfilesystem="urn:WebWorks-XSLT-Extension-FileSystem"
                              xmlns:wwuri="urn:WebWorks-XSLT-Extension-URI"
                              xmlns:wwstring="urn:WebWorks-XSLT-Extension-StringUtilities"
                              xmlns:wwunits="urn:WebWorks-XSLT-Extension-Units"
                              xmlns:wwfilesext="urn:WebWorks-XSLT-Extension-Files"
                              xmlns:wwprojext="urn:WebWorks-XSLT-Extension-Project"
                              xmlns:wwexsldoc="urn:WebWorks-XSLT-Extension-Document"
                              xmlns:wwexec="urn:WebWorks-XSLT-Extension-Execute"
                              xmlns:wwenv="urn:WebWorks-XSLT-Extension-Environment"
                              exclude-result-prefixes="xsl msxsl wwmode wwfiles wwdoc wwsplits wwproject wwprogress wwlog wwfilesystem wwuri wwstring wwunits wwfilesext wwprojext wwexsldoc wwexec wwenv"
>
 <xsl:param name="GlobalInput" />
 <xsl:param name="GlobalPipelineName" />
 <xsl:param name="GlobalProject" />
 <xsl:param name="GlobalFiles" />
 <xsl:param name="ParameterDependsType" />
 <xsl:param name="ParameterType" />
 <xsl:param name="ParameterSplitsType" />
 <xsl:param name="ParameterSearchDataDirectoryFileType" />
 <xsl:param name="ParameterJavaHelpURI" />
 <xsl:param name="ParameterCategory" />
 <xsl:param name="ParameterUse" />
 <xsl:param name="ParameterDeploy" />


 <xsl:output method="xml" encoding="UTF-8" indent="yes" />
 <xsl:strip-space elements="*" />

 <xsl:key name="wwfiles-files-by-type" match="wwfiles:File" use="@type" />
 <xsl:key name="wwfiles-files-by-groupid-type" match="wwfiles:File" use="concat(@groupID, ':', @type)" />
 <xsl:key name="wwsplits-files-by-groupid-type" match="wwsplits:File" use="concat(@groupID, ':', @type)" />


 <xsl:variable name="GlobalActionChecksum">
  <xsl:variable name="VarTransformChecksums">
   <xsl:value-of select="concat(wwuri:AsFilePath('wwtransform:self'), ':', wwfilesystem:GetChecksum(wwuri:AsFilePath('wwtransform:self')))" />
  </xsl:variable>
  <xsl:value-of select="wwstring:MD5Checksum($VarTransformChecksums)" />
 </xsl:variable>


 <xsl:template match="/">
  <wwfiles:Files version="1.0">

   <!-- Search? -->
   <!--         -->
   <xsl:if test="wwprojext:GetFormatSetting('enable-search-tab', 'true') = 'true'">
    <!-- Set JavaHelp path -->
    <!--                   -->
    <xsl:variable name="VarJavaHelpPath" select="wwprojext:GetFormatSetting('local-javahelp-location', wwuri:AsFilePath($ParameterJavaHelpURI))" />

    <xsl:for-each select="$GlobalFiles[1]">
     <xsl:variable name="VarSearchConfigurationFiles" select="key('wwfiles-files-by-type', $ParameterDependsType)" />

     <!-- Process search configuration files -->
     <!--                                    -->
     <xsl:value-of select="wwprogress:Start(count($VarSearchConfigurationFiles))" />
     <xsl:for-each select="$VarSearchConfigurationFiles">
      <xsl:variable name="VarSearchConfigurationFile" select="." />

      <xsl:value-of select="wwprogress:Start(1)" />

      <!-- Load splits -->
      <!--             -->
      <xsl:variable name="VarSplitsFile" select="key('wwfiles-files-by-groupid-type', concat($VarSearchConfigurationFile/@groupID, ':', $ParameterSplitsType))[1]" />
      <xsl:variable name="VarSplits" select="wwexsldoc:LoadXMLWithoutResolver($VarSplitsFile/@path)" />

      <!-- Determine group name -->
      <!--                      -->
      <xsl:variable name="VarGroupName" select="wwprojext:GetGroupName($VarSearchConfigurationFile/@groupID)" />

      <!-- Determine group output directory path -->
      <!--                                       -->
      <xsl:variable name="VarGroupOutputDirectoryPath" select="wwfilesystem:Combine(wwprojext:GetTargetOutputDirectoryPath(), $VarGroupName)" />

      <!-- Search data directory path -->
      <!--                            -->
      <xsl:variable name="VarSearchDataDirectoryPath">
       <xsl:for-each select="$VarSplits[1]">
        <xsl:value-of select="key('wwsplits-files-by-groupid-type', concat($VarSearchConfigurationFile/@groupID, ':', $ParameterSearchDataDirectoryFileType))[1]/@path" />
       </xsl:for-each>
      </xsl:variable>
      <xsl:variable name="VarSearchMarkerPath" select="wwfilesystem:Combine($VarSearchDataDirectoryPath, 'DOCS')" />

      <!-- Call template -->
      <!--               -->
      <xsl:variable name="VarGroupFiles" select="$GlobalFiles//wwfiles:File[@groupID = $VarSearchConfigurationFile/@groupID]" />
      <xsl:variable name="VarGroupFilesCount" select="count($VarGroupFiles)" />
      <xsl:variable name="VarUpToDate" select="wwfilesext:UpToDate($VarSearchMarkerPath, concat($GlobalProject/wwproject:Project/@ChangeID, ':', $VarGroupFilesCount), $VarSearchConfigurationFile/@groupID, '', $GlobalActionChecksum)" />
      <xsl:if test="not($VarUpToDate)">
       <xsl:variable name="VarResult">
        <xsl:call-template name="Index">
         <xsl:with-param name="ParamJavaHelpPath" select="$VarJavaHelpPath" />
         <xsl:with-param name="ParamGroupOutputDirectoryPath" select="$VarGroupOutputDirectoryPath" />
         <xsl:with-param name="ParamSearchConfigurationPath" select="$VarSearchConfigurationFile/@path" />
         <xsl:with-param name="ParamSearchDataDirectoryPath" select="$VarSearchDataDirectoryPath" />
        </xsl:call-template>
       </xsl:variable>
      </xsl:if>

      <!-- Report search files -->
      <!--                     -->
      <xsl:if test="wwfilesystem:DirectoryExists($VarSearchDataDirectoryPath)">
       <xsl:variable name="VarSearchFiles" select="wwfilesystem:GetFiles($VarSearchDataDirectoryPath)" />
       <xsl:for-each select="$VarSearchFiles/wwfiles:Files/wwfiles:File">
        <xsl:variable name="VarSearchFile" select="." />

        <wwfiles:File path="{$VarSearchFile/@path}" type="{$ParameterType}" checksum="{wwfilesystem:GetChecksum($VarSearchFile/@path)}" projectchecksum="{concat($GlobalProject/wwproject:Project/@ChangeID, ':', $VarGroupFilesCount)}" groupID="{$VarSearchConfigurationFile/@groupID}" documentID="" actionchecksum="{$GlobalActionChecksum}" category="{$ParameterCategory}" use="{$ParameterUse}" deploy="{$ParameterDeploy}">
         <xsl:if test="$VarSearchFile/@path = $VarSearchMarkerPath">
          <xsl:for-each select="$VarGroupFiles">
           <xsl:variable name="VarGroupFile" select="." />

           <xsl:if test="starts-with($VarGroupFile/@path, $VarGroupOutputDirectoryPath)">
            <wwfiles:Depends path="{$VarGroupFile/@path}" checksum="{$VarGroupFile/@checksum}" groupID="{$VarGroupFile/@groupID}" documentID="{$VarGroupFile/@documentID}" />
           </xsl:if>
          </xsl:for-each>
         </xsl:if>
        </wwfiles:File>
       </xsl:for-each>
      </xsl:if>

      <xsl:value-of select="wwprogress:End()" />
     </xsl:for-each>

     <xsl:value-of select="wwprogress:End()" />
    </xsl:for-each>
   </xsl:if>

  </wwfiles:Files>
 </xsl:template>


 <xsl:template name="Index">
  <xsl:param name="ParamJavaHelpPath" />
  <xsl:param name="ParamGroupOutputDirectoryPath" />
  <xsl:param name="ParamSearchConfigurationPath" />
  <xsl:param name="ParamSearchDataDirectoryPath" />

  <!-- Set JDK path -->
  <!--              -->
  <xsl:variable name="VarJDKPath">
   <xsl:variable name="VarJDKHomePathSetting" select="wwprojext:GetFormatSetting('local-jdk-location', wwenv:JDKHome())" />
   <xsl:choose>
    <xsl:when test="(string-length($VarJDKHomePathSetting) &gt; 0) and (wwfilesystem:DirectoryExists($VarJDKHomePathSetting))">
     <xsl:value-of select="$VarJDKHomePathSetting" />
    </xsl:when>

    <xsl:otherwise>
     <xsl:value-of select="wwenv:JDKHome()" />
    </xsl:otherwise>
   </xsl:choose>
  </xsl:variable>

  <!-- Clear stale search files -->
  <!--                          -->
  <xsl:if test="wwfilesystem:DirectoryExists($ParamSearchDataDirectoryPath)">
   <xsl:variable name="VarIgnore">
    <xsl:value-of select="wwfilesystem:DeleteDirectory($ParamSearchDataDirectoryPath)" />
   </xsl:variable>
  </xsl:if>

  <!-- Define Indexer path -->
  <!--                     -->
  <xsl:variable name="VarJHIndexerPath" select="wwfilesystem:Combine($ParamJavaHelpPath, 'javahelp', 'bin', 'jhindexer.jar')" />

  <!-- Execute Indexer Command -->
  <!--                         -->
  <xsl:variable name="VarSearchIndexerCommand" select="wwfilesystem:Combine($VarJDKPath, 'bin', 'java')" />
  <xsl:variable name="VarExecuteResult" select="wwexec:ExecuteCommandInDirectory($ParamGroupOutputDirectoryPath, $VarSearchIndexerCommand, '-mx64m', '-jar', $VarJHIndexerPath, '-verbose', '-c', $ParamSearchConfigurationPath)" />

  <!-- Log result -->
  <!--            -->
  <xsl:variable name="VarIgnore">
   <xsl:value-of select="wwlog:Message($VarExecuteResult/wwexec:Result/wwexec:Stream[@name = 'Output']/text())" />
   <xsl:if test="($VarExecuteResult/wwexec:Result/@retcode != 0) or (string-length($VarExecuteResult/wwexec:Result/wwexec:Stream[@name = 'Error']/text()) &gt; 0)">
    <xsl:value-of select="wwlog:Warning($VarExecuteResult/wwexec:Result/wwexec:Stream[@name = 'Error']/text())" />
   </xsl:if>
  </xsl:variable>
 </xsl:template>
</xsl:stylesheet>
