<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                              xmlns:msxsl="urn:schemas-microsoft-com:xslt"
                              xmlns:wwmode="urn:WebWorks-Engine-Mode"
                              xmlns:wwfiles="urn:WebWorks-Engine-Files-Schema"
                              xmlns:wwdoc="urn:WebWorks-Document-Schema"
                              xmlns:wwsplits="urn:WebWorks-Engine-Splits-Schema"
                              xmlns:wwproject="urn:WebWorks-Publish-Project"
                              xmlns:wwprogress="urn:WebWorks-XSLT-Extension-Progress"
                              xmlns:wwlog="urn:WebWorks-XSLT-Extension-Log"
                              xmlns:wwfilesystem="urn:WebWorks-XSLT-Extension-FileSystem"
                              xmlns:wwuri="urn:WebWorks-XSLT-Extension-URI"
                              xmlns:wwstring="urn:WebWorks-XSLT-Extension-StringUtilities"
                              xmlns:wwunits="urn:WebWorks-XSLT-Extension-Units"
                              xmlns:wwfilesext="urn:WebWorks-XSLT-Extension-Files"
                              xmlns:wwprojext="urn:WebWorks-XSLT-Extension-Project"
                              xmlns:wwexsldoc="urn:WebWorks-XSLT-Extension-Document"
                              xmlns:wwexec="urn:WebWorks-XSLT-Extension-Execute"
                              xmlns:wwenv="urn:WebWorks-XSLT-Extension-Environment"
                              exclude-result-prefixes="xsl msxsl wwmode wwfiles wwdoc wwsplits wwproject wwprogress wwlog wwfilesystem wwuri wwstring wwunits wwfilesext wwprojext wwexsldoc wwexec wwenv"
>
 <xsl:param name="GlobalInput" />
 <xsl:param name="GlobalPipelineName" />
 <xsl:param name="GlobalProject" />
 <xsl:param name="GlobalFiles" />
 <xsl:param name="ParameterDependsType" />
 <xsl:param name="ParameterMergeDependsType" />
 <xsl:param name="ParameterType" />
 <xsl:param name="ParameterMergeType" />
 <xsl:param name="ParameterSplitsType" />
 <xsl:param name="ParameterJarFilesDeployAttribute" />
 <xsl:param name="ParameterExecutableJarSplitFileType" />
 <xsl:param name="ParameterCategory" />
 <xsl:param name="ParameterUse" />
 <xsl:param name="ParameterDeploy" />
 <xsl:param name="ParameterMergeCategory" />
 <xsl:param name="ParameterMergeUse" />
 <xsl:param name="ParameterMergeDeploy" />


 <xsl:output method="xml" encoding="UTF-8" indent="yes" />
 <xsl:strip-space elements="*" />

 <xsl:key name="wwfiles-files-by-type" match="wwfiles:File" use="@type" />
 <xsl:key name="wwfiles-files-by-groupid-type" match="wwfiles:File" use="concat(@groupID, ':', @type)" />
 <xsl:key name="wwfiles-files-by-deploy" match="wwfiles:File" use="@deploy" />
 <xsl:key name="wwfiles-files-by-groupid-deploy" match="wwfiles:File" use="concat(@groupID, ':', @deploy)" />
 <xsl:key name="wwsplits-files-by-groupid-type" match="wwsplits:File" use="concat(@groupID, ':', @type)" />


 <xsl:variable name="GlobalActionChecksum">
  <xsl:variable name="VarTransformChecksums">
   <xsl:value-of select="concat(wwuri:AsFilePath('wwtransform:self'), ':', wwfilesystem:GetChecksum(wwuri:AsFilePath('wwtransform:self')))" />
  </xsl:variable>
  <xsl:value-of select="wwstring:MD5Checksum($VarTransformChecksums)" />
 </xsl:variable>


 <xsl:template match="/">
  <wwfiles:Files version="1.0">

   <xsl:value-of select="wwprogress:Start(2)" />

   <xsl:value-of select="wwprogress:Start(1)" />
   <!-- Iterate input documents -->
   <!--                         -->
   <xsl:for-each select="$GlobalFiles[1]">
    <xsl:variable name="VarManifestFiles" select="key('wwfiles-files-by-type', $ParameterDependsType)" />

    <xsl:value-of select="wwprogress:Start(count($VarManifestFiles))" />

    <xsl:for-each select="$VarManifestFiles">
     <xsl:variable name="VarManifestFile" select="." />

     <xsl:value-of select="wwprogress:Start(1)" />

     <!-- Aborted? -->
     <!--          -->
     <xsl:if test="not(wwprogress:Abort())">
      <!-- Load splits -->
      <!--             -->
      <xsl:variable name="VarSplitsFile" select="key('wwfiles-files-by-groupid-type', concat($VarManifestFile/@groupID, ':', $ParameterSplitsType))[1]" />
      <xsl:variable name="VarSplits" select="wwexsldoc:LoadXMLWithoutResolver($VarSplitsFile/@path)" />

      <!-- Determine group name -->
      <!--                      -->
      <xsl:variable name="VarGroupName" select="wwprojext:GetGroupName($VarManifestFile/@groupID)" />

      <!-- Jar Path -->
      <!--          -->
      <xsl:variable name="VarJarPath">
       <xsl:for-each select="$VarSplits[1]">
        <xsl:value-of select="key('wwsplits-files-by-groupid-type', concat($VarManifestFile/@groupID, ':', $ParameterExecutableJarSplitFileType))[1]/@path" />
       </xsl:for-each>
      </xsl:variable>

      <!-- Jar Files -->
      <!--           -->
      <xsl:for-each select="$GlobalFiles[1]">
       <xsl:variable name="VarJarFiles" select="key('wwfiles-files-by-groupid-deploy', concat($VarManifestFile/@groupID, ':', $ParameterJarFilesDeployAttribute))" />
       <xsl:variable name="VarJarFilesChecksum">
        <xsl:call-template name="FilesChecksum">
         <xsl:with-param name="ParamFiles" select="$VarJarFiles" />
        </xsl:call-template>
       </xsl:variable>

       <!-- Call template -->
       <!--               -->
       <xsl:variable name="VarUpToDate" select="wwfilesext:UpToDate($VarJarPath, concat($GlobalProject/wwproject:Project/@ChangeID, ':', $VarJarFilesChecksum), $VarManifestFile/@groupID, '', $GlobalActionChecksum)" />
       <xsl:if test="not($VarUpToDate)">
        <xsl:variable name="VarResult">
         <xsl:call-template name="Jar">
          <xsl:with-param name="ParamExecutionDirectory" select="wwfilesystem:GetDirectoryName($VarManifestFile/@path)" />
          <xsl:with-param name="ParamManifestPath" select="$VarManifestFile/@path" />
          <xsl:with-param name="ParamJarPath" select="$VarJarPath" />
          <xsl:with-param name="ParamJarFiles" select="$VarJarFiles" />
         </xsl:call-template>
        </xsl:variable>
       </xsl:if>

       <!-- Aborted? -->
       <!--          -->
       <xsl:if test="not(wwprogress:Abort())">
        <wwfiles:File path="{$VarJarPath}" type="{$ParameterType}" checksum="{wwfilesystem:GetChecksum($VarJarPath)}" projectchecksum="{concat($GlobalProject/wwproject:Project/@ChangeID, ':', $VarJarFilesChecksum)}" groupID="{$VarManifestFile/@groupID}" documentID="" actionchecksum="{$GlobalActionChecksum}" category="{$ParameterCategory}" use="{$ParameterUse}" deploy="{$ParameterDeploy}">
         <wwfiles:Depends path="{$VarManifestFile/@path}" checksum="{wwfilesystem:GetChecksum($VarManifestFile/@path)}" groupID="{$VarManifestFile/@groupID}" documentID="{$VarManifestFile/@documentID}" />

         <xsl:call-template name="FilesAsDepends">
          <xsl:with-param name="ParamFiles" select="$VarJarFiles" />
         </xsl:call-template>
        </wwfiles:File>
       </xsl:if>
      </xsl:for-each>
     </xsl:if>

     <xsl:value-of select="wwprogress:End()" />
    </xsl:for-each>

    <xsl:value-of select="wwprogress:End()" />
   </xsl:for-each>
   <xsl:value-of select="wwprogress:End()" />

   <!-- Merge -->
   <!--       -->
   <xsl:value-of select="wwprogress:Start(1)" />
   <xsl:variable name="VarProjectGroups" select="$GlobalProject/wwproject:Project/wwproject:Groups/wwproject:Group" />
   <xsl:if test="count($VarProjectGroups) &gt; 1">
    <xsl:for-each select="$GlobalFiles[1]">
     <xsl:variable name="VarMergeHelpSetFile" select="key('wwfiles-files-by-type', $ParameterMergeDependsType)[1]" />
     <xsl:if test="count($VarMergeHelpSetFile) = 1">
      <xsl:variable name="VarMergePath" select="wwfilesystem:GetWithExtensionReplaced($VarMergeHelpSetFile/@path, '.jar')" />

      <!-- Aborted? -->
      <!--          -->
      <xsl:if test="not(wwprogress:Abort())">
       <!-- Jar Files -->
       <!--           -->
       <xsl:for-each select="$GlobalFiles[1]">
        <xsl:variable name="VarJarFiles" select="key('wwfiles-files-by-deploy', $ParameterJarFilesDeployAttribute)" />
        <xsl:variable name="VarJarFilesChecksum">
         <xsl:call-template name="FilesChecksum">
          <xsl:with-param name="ParamFiles" select="$VarJarFiles" />
         </xsl:call-template>
        </xsl:variable>

        <!-- Call template -->
        <!--               -->
        <xsl:variable name="VarUpToDate" select="wwfilesext:UpToDate($VarMergePath, concat($GlobalProject/wwproject:Project/@ChangeID, ':', $VarJarFilesChecksum), '', '', $GlobalActionChecksum)" />
        <xsl:if test="not($VarUpToDate)">
         <xsl:variable name="VarResult">
          <xsl:call-template name="Jar">
           <xsl:with-param name="ParamExecutionDirectory" select="wwfilesystem:GetDirectoryName($VarMergeHelpSetFile/@path)" />
           <xsl:with-param name="ParamManifestPath" select="$VarMergeHelpSetFile/@path" />
           <xsl:with-param name="ParamJarPath" select="$VarMergePath" />
           <xsl:with-param name="ParamJarFiles" select="$VarJarFiles" />
          </xsl:call-template>
         </xsl:variable>
        </xsl:if>

        <!-- Aborted? -->
        <!--          -->
        <xsl:if test="not(wwprogress:Abort())">
         <wwfiles:File path="{$VarMergePath}" type="{$ParameterMergeType}" checksum="{wwfilesystem:GetChecksum($VarMergePath)}" projectchecksum="{concat($GlobalProject/wwproject:Project/@ChangeID, ':', $VarJarFilesChecksum)}" groupID="" documentID="" actionchecksum="{$GlobalActionChecksum}" category="{$ParameterMergeCategory}" use="{$ParameterMergeUse}" deploy="{$ParameterMergeDeploy}">
          <wwfiles:Depends path="{$VarMergeHelpSetFile/@path}" checksum="{$VarMergeHelpSetFile/@checksum}" groupID="{$VarMergeHelpSetFile/@groupID}" documentID="{$VarMergeHelpSetFile/@documentID}" />

          <xsl:call-template name="FilesAsDepends">
           <xsl:with-param name="ParamFiles" select="$VarJarFiles" />
          </xsl:call-template>
         </wwfiles:File>
        </xsl:if>
       </xsl:for-each>
      </xsl:if>
     </xsl:if>
    </xsl:for-each>
   </xsl:if>
   <xsl:value-of select="wwprogress:End()" />

   <xsl:value-of select="wwprogress:End()" />

  </wwfiles:Files>
 </xsl:template>


 <xsl:template name="FilesChecksum">
  <xsl:param name="ParamFiles" />

  <xsl:variable name="VarFilePaths">
   <xsl:for-each select="$ParamFiles">
    <xsl:variable name="VarFile" select="." />

    <xsl:value-of select="$VarFile/@path" />
    <xsl:text>:</xsl:text>
   </xsl:for-each>
  </xsl:variable>

  <xsl:value-of select="wwstring:MD5Checksum($VarFilePaths)" />
 </xsl:template>


 <xsl:template name="FilesAsDepends">
  <xsl:param name="ParamFiles" />

  <xsl:for-each select="$ParamFiles">
   <xsl:variable name="VarFile" select="." />

   <wwfiles:Depends path="{$VarFile/@path}" checksum="{$VarFile/@checksum}" groupID="{$VarFile/@groupID}" documentID="{$VarFile/@documentID}" />
  </xsl:for-each>
 </xsl:template>


 <xsl:template name="Jar">
  <xsl:param name="ParamExecutionDirectory" />
  <xsl:param name="ParamManifestPath" />
  <xsl:param name="ParamJarPath" />
  <xsl:param name="ParamJarFiles" />

  <!-- Set JDK path -->
  <!--              -->
  <xsl:variable name="VarJDKPath">
   <xsl:variable name="VarJDKHomePathSetting" select="wwprojext:GetFormatSetting('local-jdk-location', wwenv:JDKHome())" />
   <xsl:choose>
    <xsl:when test="(string-length($VarJDKHomePathSetting) &gt; 0) and (wwfilesystem:DirectoryExists($VarJDKHomePathSetting))">
     <xsl:value-of select="$VarJDKHomePathSetting" />
    </xsl:when>

    <xsl:otherwise>
     <xsl:value-of select="wwenv:JDKHome()" />
    </xsl:otherwise>
   </xsl:choose>
  </xsl:variable>

  <!-- Execute Jar Command -->
  <!--                     -->
  <xsl:variable name="VarJarProgram" select="wwfilesystem:Combine($VarJDKPath, 'bin', 'jar')" />
  <xsl:variable name="VarJarResult" select="wwexec:ExecuteCommandInDirectory($ParamExecutionDirectory, $VarJarProgram, 'cvfm', $ParamJarPath, $ParamManifestPath)" />

  <!-- Log result -->
  <!--            -->
  <xsl:variable name="VarLogJarResult">
   <xsl:value-of select="wwlog:Message($VarJarResult/wwexec:Result/wwexec:Stream[@name = 'Output']/text())" />
   <xsl:if test="($VarJarResult/wwexec:Result/@retcode != 0) or (string-length($VarJarResult/wwexec:Result/wwexec:Stream[@name = 'Error']/text()) &gt; 0)">
    <xsl:value-of select="wwlog:Warning($VarJarResult/wwexec:Result/wwexec:Stream[@name = 'Error']/text())" />
   </xsl:if>
  </xsl:variable>

  <!-- Add files -->
  <!--           -->
  <xsl:if test="$VarJarResult/wwexec:Result/@retcode = 0">
   <!-- Segment interval -->
   <!--                  -->
   <xsl:variable name="VarInterval" select="number(32)" />

   <!-- Reference path for get relative path calls -->
   <!--                                            -->
   <xsl:variable name="VarReferencePath" select="wwfilesystem:Combine($ParamExecutionDirectory, 'dummy.component')" />

   <xsl:for-each select="$ParamJarFiles">
    <!-- Break up into segments to avoid CLI command length limits -->
    <!--                                                           -->
    <xsl:if test="((position() - 1) mod $VarInterval) = 0">
     <!-- Record start/end positions -->
     <!--                            -->
     <xsl:variable name="VarStartPosition" select="position()" />
     <xsl:variable name="VarEndPosition" select="position() + $VarInterval - 1" />

     <!-- Aborted? -->
     <!--          -->
     <xsl:if test="not(wwprogress:Abort())">
      <!-- Jar Arguments -->
      <!--               -->
      <xsl:variable name="VarArguments">
       <!-- Switches -->
       <!--          -->
       <xsl:text>&quot;uf&quot;</xsl:text>

       <!-- Jar File -->
       <!--          -->
       <xsl:text> &quot;</xsl:text>
       <xsl:value-of select="$ParamJarPath" />
       <xsl:text>&quot;</xsl:text>

       <!-- Files to add -->
       <!--              -->
       <xsl:variable name="VarJarFilesToAdd" select="$ParamJarFiles[(position() &gt;= $VarStartPosition) and (position() &lt;= $VarEndPosition)]" />
       <xsl:for-each select="$VarJarFilesToAdd">
        <xsl:variable name="VarJarFileToAdd" select="." />

        <xsl:variable name="VarRelativePath" select="wwfilesystem:GetRelativeTo($VarJarFileToAdd/@path, $VarReferencePath)" />

        <xsl:text> &quot;</xsl:text>
        <xsl:value-of select="$VarRelativePath" />
        <xsl:text>&quot;</xsl:text>
       </xsl:for-each>
      </xsl:variable>

      <!-- Execute Jar Add Command -->
      <!--                         -->
      <xsl:variable name="VarJarAddResult" select="wwexec:ExecuteProgramWithArgumentsInDirectory($ParamExecutionDirectory, $VarJarProgram, $VarArguments)" />

      <!-- Log result -->
      <!--            -->
      <xsl:variable name="VarLogJarAddResult">
       <xsl:if test="string-length($VarJarAddResult/wwexec:Result/wwexec:Stream[@name = 'Output']/text()) &gt; 0">
        <xsl:value-of select="wwlog:Message($VarJarAddResult/wwexec:Result/wwexec:Stream[@name = 'Output']/text())" />
       </xsl:if>
       <xsl:if test="($VarJarAddResult/wwexec:Result/@retcode != 0) or (string-length($VarJarAddResult/wwexec:Result/wwexec:Stream[@name = 'Error']/text()) &gt; 0)">
        <xsl:value-of select="wwlog:Warning($VarJarAddResult/wwexec:Result/wwexec:Stream[@name = 'Error']/text())" />
       </xsl:if>
      </xsl:variable>
     </xsl:if>
    </xsl:if>
   </xsl:for-each>
  </xsl:if>
 </xsl:template>
</xsl:stylesheet>
