<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                              xmlns:msxsl="urn:schemas-microsoft-com:xslt"
                              xmlns:wwmode="urn:WebWorks-Engine-Mode"
                              xmlns:wwfiles="urn:WebWorks-Engine-Files-Schema"
                              xmlns:wwdoc="urn:WebWorks-Document-Schema"
                              xmlns:wwsplits="urn:WebWorks-Engine-Splits-Schema"
                              xmlns:wwproject="urn:WebWorks-Publish-Project"
                              xmlns:wwprogress="urn:WebWorks-XSLT-Extension-Progress"
                              xmlns:wwlog="urn:WebWorks-XSLT-Extension-Log"
                              xmlns:wwfilesystem="urn:WebWorks-XSLT-Extension-FileSystem"
                              xmlns:wwuri="urn:WebWorks-XSLT-Extension-URI"
                              xmlns:wwstring="urn:WebWorks-XSLT-Extension-StringUtilities"
                              xmlns:wwunits="urn:WebWorks-XSLT-Extension-Units"
                              xmlns:wwfilesext="urn:WebWorks-XSLT-Extension-Files"
                              xmlns:wwprojext="urn:WebWorks-XSLT-Extension-Project"
                              xmlns:wwexsldoc="urn:WebWorks-XSLT-Extension-Document"
                              exclude-result-prefixes="xsl msxsl wwmode wwfiles wwdoc wwsplits wwproject wwprogress wwlog wwfilesystem wwuri wwstring wwunits wwfilesext wwprojext wwexsldoc"
>
 <xsl:param name="GlobalInput" />
 <xsl:param name="GlobalPipelineName" />
 <xsl:param name="GlobalProject" />
 <xsl:param name="GlobalFiles" />
 <xsl:param name="ParameterDependsType" />
 <xsl:param name="ParameterMergeDependsType" />
 <xsl:param name="ParameterSplitsType" />
 <xsl:param name="ParameterType" />
 <xsl:param name="ParameterMergeType" />
 <xsl:param name="ParameterSplitFileType" />


 <xsl:output method="xml" encoding="UTF-8" indent="yes" />
 <xsl:strip-space elements="*" />

 <xsl:key name="wwfiles-files-by-type" match="wwfiles:File" use="@type" />
 <xsl:key name="wwfiles-files-by-groupid-type" match="wwfiles:File" use="concat(@groupID, ':', @type)" />
 <xsl:key name="wwsplits-files-by-groupid-type" match="wwsplits:File" use="concat(@groupID, ':', @type)" />


 <xsl:variable name="GlobalActionChecksum">
  <xsl:variable name="VarTransformChecksums">
   <xsl:value-of select="concat(wwuri:AsFilePath('wwtransform:self'), ':', wwfilesystem:GetChecksum(wwuri:AsFilePath('wwtransform:self')))" />
  </xsl:variable>
  <xsl:value-of select="wwstring:MD5Checksum($VarTransformChecksums)" />
 </xsl:variable>


 <xsl:template match="/">
  <wwfiles:Files version="1.0">

   <!-- Groups -->
   <!--        -->
   <xsl:variable name="VarProjectGroups" select="$GlobalProject/wwproject:Project/wwproject:Groups/wwproject:Group" />
   <xsl:value-of select="wwprogress:Start(count($VarProjectGroups) + 1)" />
   <xsl:for-each select="$VarProjectGroups">
    <xsl:variable name="VarProjectGroup" select="." />

    <xsl:value-of select="wwprogress:Start(1)" />

    <!-- Access splits -->
    <!--               -->
    <xsl:for-each select="$GlobalFiles[1]">
     <xsl:variable name="VarFilesNameInfo" select="key('wwfiles-files-by-groupid-type', concat($VarProjectGroup/@GroupID, ':', $ParameterSplitsType))" />

     <xsl:value-of select="wwprogress:Start(1)" />

     <xsl:for-each select="$VarFilesNameInfo[1]">
      <xsl:variable name="VarSplitsFileInfo" select="." />

      <xsl:value-of select="wwprogress:Start(1)"/>

      <!-- Load splits -->
      <!--             -->
      <xsl:variable name="VarSplits" select="wwexsldoc:LoadXMLWithoutResolver($VarSplitsFileInfo/@path)" />

      <!-- Iterate input documents -->
      <!--                         -->
      <xsl:for-each select="$GlobalFiles[1]">
       <xsl:variable name="VarFilesByType" select="key('wwfiles-files-by-groupid-type', concat($VarProjectGroup/@GroupID, ':', $ParameterDependsType))" />

       <xsl:value-of select="wwprogress:Start(count($VarFilesByType))" />

       <xsl:for-each select="$VarFilesByType[1]">
        <xsl:value-of select="wwprogress:Start(1)" />

        <xsl:variable name="VarFilesHelpSetNode" select="." />
        <xsl:variable name="VarHelpSetPath" select="$VarFilesHelpSetNode/@path" />

         <!-- Output Path -->
         <!--             -->
         <xsl:variable name="VarPath">
          <xsl:for-each select="$VarSplits[1]">
           <xsl:value-of select="key('wwsplits-files-by-groupid-type', concat($VarFilesHelpSetNode/@groupID, ':', $ParameterSplitFileType))[1]/@path" />
          </xsl:for-each>
         </xsl:variable>

        <!-- Call template -->
        <!--               -->
        <xsl:variable name="VarUpToDate" select="wwfilesext:UpToDate($VarPath, $GlobalProject/wwproject:Project/@ChangeID, @groupID, '', $GlobalActionChecksum)" />
        <xsl:if test="not($VarUpToDate)">
         <xsl:variable name="VarResultAsXML">
          <xsl:call-template name="Manifest">
           <xsl:with-param name="ParamHelpSetPath" select="wwfilesystem:GetFileName($VarHelpSetPath)" />
          </xsl:call-template>
         </xsl:variable>
         <xsl:variable name="VarResult" select="msxsl:node-set($VarResultAsXML)" />
         <xsl:variable name="VarWriteResult" select="wwexsldoc:Document($VarResult, $VarPath, wwprojext:GetFormatSetting('encoding', 'utf-8'), 'text')" />
        </xsl:if>

        <wwfiles:File path="{$VarPath}" type="{$ParameterType}" checksum="{wwfilesystem:GetChecksum($VarPath)}" projectchecksum="{$GlobalProject/wwproject:Project/@ChangeID}" groupID="{$VarFilesHelpSetNode/@groupID}" documentID="" actionchecksum="{$GlobalActionChecksum}">
         <wwfiles:Depends path="{$VarFilesHelpSetNode/@path}" checksum="{$VarFilesHelpSetNode/@checksum}" groupID="{$VarFilesHelpSetNode/@groupID}" documentID="{$VarFilesHelpSetNode/@documentID}" />
        </wwfiles:File>

        <xsl:value-of select="wwprogress:End()" />
       </xsl:for-each>

       <xsl:value-of select="wwprogress:End()" />
      </xsl:for-each>

      <xsl:value-of select="wwprogress:End()" />
     </xsl:for-each>

     <xsl:value-of select="wwprogress:End()" />
    </xsl:for-each>

    <xsl:value-of select="wwprogress:End()" />
   </xsl:for-each>

   <!-- Merge -->
   <!--       -->
   <xsl:value-of select="wwprogress:Start(1)" />
   <xsl:if test="count($VarProjectGroups) &gt; 1">
    <xsl:for-each select="$GlobalFiles[1]">
     <xsl:variable name="VarMergeHelpSetFile" select="key('wwfiles-files-by-type', $ParameterMergeDependsType)[1]" />
     <xsl:if test="count($VarMergeHelpSetFile) = 1">
      <xsl:variable name="VarMergePath" select="wwfilesystem:GetWithExtensionReplaced($VarMergeHelpSetFile/@path, '.mft')" />

      <!-- Call template -->
      <!--               -->
      <xsl:variable name="VarUpToDate" select="wwfilesext:UpToDate($VarMergePath, $GlobalProject/wwproject:Project/@ChangeID, '', '', $GlobalActionChecksum)" />
      <xsl:if test="not($VarUpToDate)">
       <xsl:variable name="VarResultAsXML">
        <xsl:call-template name="Manifest">
         <xsl:with-param name="ParamHelpSetPath" select="wwuri:Unescape(wwuri:GetRelativeTo($VarMergeHelpSetFile/@path, wwfilesystem:Combine(wwprojext:GetTargetOutputDirectoryPath(), 'dummy.component')))" />
        </xsl:call-template>
       </xsl:variable>
       <xsl:variable name="VarResult" select="msxsl:node-set($VarResultAsXML)" />
       <xsl:variable name="VarWriteResult" select="wwexsldoc:Document($VarResult, $VarMergePath, wwprojext:GetFormatSetting('encoding', 'utf-8'), 'text')" />
      </xsl:if>

      <wwfiles:File path="{$VarMergePath}" type="{$ParameterMergeType}" checksum="{wwfilesystem:GetChecksum($VarMergePath)}" projectchecksum="{$GlobalProject/wwproject:Project/@ChangeID}" groupID="" documentID="" actionchecksum="{$GlobalActionChecksum}">
       <wwfiles:Depends path="{$VarMergeHelpSetFile/@path}" checksum="{$VarMergeHelpSetFile/@checksum}" groupID="{$VarMergeHelpSetFile/@groupID}" documentID="{$VarMergeHelpSetFile/@documentID}" />
      </wwfiles:File>
     </xsl:if>
    </xsl:for-each>
   </xsl:if>
   <xsl:value-of select="wwprogress:End()" />

   <xsl:value-of select="wwprogress:End()" />

  </wwfiles:Files>
 </xsl:template>

 <xsl:template name="Manifest">
  <xsl:param name="ParamHelpSetPath" />

  <xsl:text>Class-Path: runjar.jar jh.jar hsviewer.jar .
</xsl:text>
  <xsl:text>Arguments: -helpset "</xsl:text><xsl:value-of select="$ParamHelpSetPath" /><xsl:text>"
</xsl:text>
  <xsl:text>Run-Class: sunw.demo.jhdemo.JHLauncher
</xsl:text>
  <xsl:text>Main-Class: com.webworks.runjar
</xsl:text>
 </xsl:template>
</xsl:stylesheet>
