<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns="http://java.sun.com/products/javahelp/map_2_0.dtd"
                              xmlns:javahelpmap="http://java.sun.com/products/javahelp/map_2_0.dtd"
                              xmlns:wwmap="urn:WebWorks-Map-Node-Schema"
                              xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                              xmlns:msxsl="urn:schemas-microsoft-com:xslt"
                              xmlns:wwmode="urn:WebWorks-Engine-Mode"
                              xmlns:wwlinks="urn:WebWorks-Engine-Links-Schema"
                              xmlns:wwfiles="urn:WebWorks-Engine-Files-Schema"
                              xmlns:wwdoc="urn:WebWorks-Document-Schema"
                              xmlns:wwbehaviors="urn:WebWorks-Behaviors-Schema"
                              xmlns:wwsplits="urn:WebWorks-Engine-Splits-Schema"
                              xmlns:wwproject="urn:WebWorks-Publish-Project"
                              xmlns:wwprogress="urn:WebWorks-XSLT-Extension-Progress"
                              xmlns:wwlog="urn:WebWorks-XSLT-Extension-Log"
                              xmlns:wwfilesystem="urn:WebWorks-XSLT-Extension-FileSystem"
                              xmlns:wwuri="urn:WebWorks-XSLT-Extension-URI"
                              xmlns:wwstring="urn:WebWorks-XSLT-Extension-StringUtilities"
                              xmlns:wwunits="urn:WebWorks-XSLT-Extension-Units"
                              xmlns:wwfilesext="urn:WebWorks-XSLT-Extension-Files"
                              xmlns:wwprojext="urn:WebWorks-XSLT-Extension-Project"
                              xmlns:wwexsldoc="urn:WebWorks-XSLT-Extension-Document"
                              exclude-result-prefixes="wwmap xsl msxsl wwmode wwlinks wwfiles wwdoc wwbehaviors wwsplits wwproject wwprogress wwlog wwfilesystem wwuri wwstring wwunits wwfilesext wwprojext wwexsldoc"
>
 <xsl:param name="GlobalInput" />
 <xsl:param name="GlobalPipelineName" />
 <xsl:param name="GlobalProject" />
 <xsl:param name="GlobalFiles" />
 <xsl:param name="ParameterDependsType" />
 <xsl:param name="ParameterLinksType" />
 <xsl:param name="ParameterSplitsType" />
 <xsl:param name="ParameterBehaviorsType" />
 <xsl:param name="ParameterGlossaryType" />
 <xsl:param name="ParameterType" />
 <xsl:param name="ParameterSplitFileType" />
 <xsl:param name="ParameterCategory" />
 <xsl:param name="ParameterUse" />
 <xsl:param name="ParameterDeploy" />


 <xsl:output method="xml" encoding="UTF-8" indent="yes" />
 <xsl:namespace-alias stylesheet-prefix="javahelpmap" result-prefix="#default" />
 <xsl:strip-space elements="*" />

 <xsl:key name="wwfiles-files-by-type" match="wwfiles:File" use="@type" />
 <xsl:key name="wwfiles-files-by-groupid-type" match="wwfiles:File" use="concat(@groupID, ':', @type)" />
 <xsl:key name="wwlinks-files-by-documentid" match="wwlinks:File" use="@documentID" />
 <xsl:key name="wwmap-image-by-path" match="wwmap:Image" use="@path" />
 <xsl:key name="wwsplits-files-by-groupid-type" match="wwsplits:File" use="concat(@groupID, ':', @type)" />

 <xsl:variable name="GlobalActionChecksum">
  <xsl:variable name="VarTransformChecksums">
   <xsl:value-of select="concat(wwuri:AsFilePath('wwtransform:self'), ':', wwfilesystem:GetChecksum(wwuri:AsFilePath('wwtransform:self')))" />
  </xsl:variable>
  <xsl:value-of select="wwstring:MD5Checksum($VarTransformChecksums)" />
 </xsl:variable>


 <xsl:template match="/">
  <wwfiles:Files version="1.0">

    <!-- Iterate input documents -->
    <!--                         -->
   <xsl:for-each select="$GlobalInput[1]">
    <!-- Load project links -->
    <!--                    -->
    <xsl:variable name="VarLinksPath" select="key('wwfiles-files-by-type', $ParameterLinksType)/@path" />
    <xsl:variable name="VarLinks" select="wwexsldoc:LoadXMLWithoutResolver($VarLinksPath)" />

    <!-- Groups -->
    <!--        -->
    <xsl:variable name="VarProjectGroups" select="$GlobalProject/wwproject:Project/wwproject:Groups/wwproject:Group" />
    <xsl:value-of select="wwprogress:Start(count($VarProjectGroups))" />
    <xsl:for-each select="$VarProjectGroups">
     <xsl:variable name="VarProjectGroup" select="." />
     <xsl:variable name="VarGroupDocuments" select="$VarProjectGroup//wwproject:Document" />

     <xsl:value-of select="wwprogress:Start(1)" />

     <!-- Access splits -->
     <!--               -->
     <xsl:for-each select="$GlobalFiles[1]">

      <xsl:value-of select="wwprogress:Start(1)" />

      <xsl:variable name="VarFilesNameInfo" select="key('wwfiles-files-by-groupid-type', concat($VarProjectGroup/@GroupID, ':', $ParameterSplitsType))" />
      <xsl:for-each select="$VarFilesNameInfo[1]">
       <xsl:variable name="VarSplitsFileInfo" select="." />

       <xsl:value-of select="wwprogress:Start(1)" />

       <!-- Load splits -->
       <!--             -->
       <xsl:variable name="VarSplits" select="wwexsldoc:LoadXMLWithoutResolver($VarSplitsFileInfo/@path)" />

       <!-- Iterate input documents -->
       <!--                         -->
       <xsl:for-each select="$GlobalFiles[1]">

        <xsl:value-of select="wwprogress:Start(1)" />

        <xsl:variable name="VarFilesByType" select="key('wwfiles-files-by-groupid-type', concat($VarProjectGroup/@GroupID, ':', $ParameterDependsType))" />
        <xsl:variable name="VarBehaviorsByType" select="key('wwfiles-files-by-groupid-type', concat($VarProjectGroup/@GroupID, ':', $ParameterBehaviorsType))" />
        <xsl:variable name="VarGlossaryByType" select="key('wwfiles-files-by-groupid-type', concat($VarProjectGroup/@GroupID, ':', $ParameterGlossaryType))" />

        <!-- Output Path -->
        <!--             -->
        <xsl:variable name="VarPath">
         <xsl:for-each select="$VarSplits[1]">
          <xsl:value-of select="key('wwsplits-files-by-groupid-type', concat($VarProjectGroup/@GroupID, ':', $ParameterSplitFileType))[1]/@path" />
         </xsl:for-each>
        </xsl:variable>

        <!-- Call template -->
        <!--               -->
        <xsl:variable name="VarUpToDate" select="wwfilesext:UpToDate($VarPath, concat($GlobalProject/wwproject:Project/@ChangeID, ':', count($VarFilesByType)), $VarProjectGroup/@GroupID, '', $GlobalActionChecksum)" />
        <xsl:if test="not($VarUpToDate)">
         <xsl:variable name="VarResultAsXML">
          <javahelpmap:map version="2.0">
           <xsl:call-template name="MapFile">
            <xsl:with-param name="ParamLinks" select="$VarLinks" />
            <xsl:with-param name="ParamGroupDocuments" select="$VarGroupDocuments" />
            <xsl:with-param name="ParamFilesByType" select="$VarFilesByType" />
            <xsl:with-param name="ParamBehaviorsByType" select="$VarBehaviorsByType" />
            <xsl:with-param name="ParamGlossaryByType" select="$VarGlossaryByType" />
            <xsl:with-param name="ParamSplitsDocument" select="$VarSplits" />
            <xsl:with-param name="ParamMapPath" select="$VarPath" />
           </xsl:call-template>
          </javahelpmap:map>
         </xsl:variable>
         <xsl:variable name="VarResult" select="msxsl:node-set($VarResultAsXML)" />
         <xsl:variable name="VarWriteResult" select="wwexsldoc:Document($VarResult, $VarPath, wwprojext:GetFormatSetting('encoding', 'utf-8'), 'xml', '1.0', 'yes', 'no', '', '-//Sun Microsystems Inc.//DTD JavaHelp Map Version 2.0//EN', 'http://java.sun.com/products/javahelp/map_2_0.dtd', '', 'text/xml')" />
        </xsl:if>

        <wwfiles:File path="{$VarPath}" type="{$ParameterType}" checksum="{wwfilesystem:GetChecksum($VarPath)}" projectchecksum="{concat($GlobalProject/wwproject:Project/@ChangeID, ':', count($VarFilesByType))}" groupID="{$VarProjectGroup/@GroupID}" documentID="" actionchecksum="{$GlobalActionChecksum}" category="{$ParameterCategory}" use="{$ParameterUse}" deploy="{$ParameterDeploy}">
         <xsl:for-each select="$VarFilesByType">
          <wwfiles:Depends path="{./@path}" checksum="{./@checksum}" groupID="{./@groupID}" documentID="{./@documentID}" />
         </xsl:for-each>
         <wwfiles:Depends path="{$VarSplitsFileInfo/@path}" checksum="{$VarSplitsFileInfo/@checksum}" groupID="{$VarSplitsFileInfo/@groupID}" documentID="{$VarSplitsFileInfo/@documentID}" />
        </wwfiles:File>

        <xsl:value-of select="wwprogress:End()" />
       </xsl:for-each>

       <xsl:value-of select="wwprogress:End()" />
      </xsl:for-each>

      <xsl:value-of select="wwprogress:End()" />
     </xsl:for-each>

     <xsl:value-of select="wwprogress:End()" />
    </xsl:for-each>

    <xsl:value-of select="wwprogress:End()" />
   </xsl:for-each>

  </wwfiles:Files>
 </xsl:template>


 <xsl:template name="MapFile">
  <xsl:param name="ParamLinks" />
  <xsl:param name="ParamGroupDocuments" />
  <xsl:param name="ParamFilesByType" />
  <xsl:param name="ParamBehaviorsByType" />
  <xsl:param name="ParamGlossaryByType" />
  <xsl:param name="ParamSplitsDocument" />
  <xsl:param name="ParamMapPath" />

  <xsl:for-each select="$ParamGroupDocuments">
   <xsl:variable name="VarGroupDocument" select="." />

   <xsl:for-each select="$ParamFilesByType[1]">
    <xsl:variable name="VarDocumentPath" select="$ParamFilesByType[@documentID = $VarGroupDocument/@DocumentID]/@path" />
    <xsl:variable name="VarDocument" select="wwexsldoc:LoadXMLWithoutResolver($VarDocumentPath)" />

    <xsl:for-each select="$ParamSplitsDocument[1]">
     <xsl:variable name="VarSplits" select="$ParamSplitsDocument//wwsplits:Split[@documentID = $VarGroupDocument/@DocumentID]" />

     <xsl:comment> Paragraph mappings for <xsl:value-of select="wwfilesystem:GetFileName($VarGroupDocument/@Path)" />&#160;</xsl:comment>
     <xsl:for-each select="$VarDocument/wwdoc:Document/wwdoc:Content/*">
      <xsl:variable name="VarContextNode" select="." />

      <xsl:choose>
       <xsl:when test="name($VarContextNode) = 'Table'">
        <xsl:variable name="VarTablePosition" select="position()" />
        <xsl:variable name="VarTableOutputPage" select="$VarSplits[($VarTablePosition &gt;= @documentstartposition) and ($VarTablePosition &lt;= @documentendposition)]/@path" />

        <xsl:for-each select="$VarContextNode//wwdoc:Paragraph">
         <xsl:variable name="VarContextParagraph" select="." />
         <javahelpmap:mapID>
          <xsl:attribute name="target">
           <xsl:value-of select="$VarGroupDocument/@DocumentID" />
           <xsl:text>_</xsl:text>
           <xsl:value-of select="$VarContextParagraph/@id" />
          </xsl:attribute>
          <xsl:attribute name="url">
           <xsl:value-of select="wwuri:Unescape(wwuri:GetRelativeTo($VarTableOutputPage, $ParamMapPath))" />
           <xsl:text>#</xsl:text>
           <xsl:value-of select="$VarContextParagraph/@id" />
          </xsl:attribute>
         </javahelpmap:mapID>
        </xsl:for-each>
       </xsl:when>

       <xsl:otherwise>
        <xsl:variable name="VarParaPosition" select="position()" />

        <xsl:variable name="VarParaOutputPage" select="$VarSplits[($VarParaPosition &gt;= @documentstartposition) and ($VarParaPosition &lt;= @documentendposition)]/@path" />

        <xsl:if test="string-length($VarParaOutputPage) &gt; 0">
         <javahelpmap:mapID>
          <xsl:attribute name="target">
           <xsl:value-of select="$VarGroupDocument/@DocumentID" />
           <xsl:text>_</xsl:text>
           <xsl:value-of select="$VarContextNode/@id" />
          </xsl:attribute>
          <xsl:attribute name="url">
           <xsl:value-of select="wwuri:Unescape(wwuri:GetRelativeTo($VarParaOutputPage, $ParamMapPath))" />
           <xsl:text>#</xsl:text>
           <xsl:value-of select="$VarContextNode/@id" />
          </xsl:attribute>
         </javahelpmap:mapID>
        </xsl:if>
       </xsl:otherwise>

      </xsl:choose>

     </xsl:for-each>
    </xsl:for-each>
   </xsl:for-each>

   <xsl:comment>Begin topic mapping</xsl:comment>

   <xsl:for-each select="$ParamLinks[1]">
    <xsl:variable name="VarLinksFiles" select="key('wwlinks-files-by-documentid', $VarGroupDocument/@DocumentID)" />

    <xsl:for-each select="$VarLinksFiles">
     <xsl:variable name="VarLinksFile" select="." />

     <xsl:for-each select="$VarLinksFile/wwlinks:Paragraph[(string-length(@topic) &gt; 0)]">
      <javahelpmap:mapID target="{@topic}">
       <xsl:attribute name="url">
        <xsl:value-of select="wwuri:Unescape(wwuri:GetRelativeTo($VarLinksFile/@path, $ParamMapPath))" />
        <xsl:text>#</xsl:text>
        <xsl:value-of select="@id" />
       </xsl:attribute>
      </javahelpmap:mapID>
     </xsl:for-each>
    </xsl:for-each>
   </xsl:for-each>

   <xsl:comment>End topic mapping</xsl:comment>

   <!-- TOC Icons -->
   <!--           -->
   <xsl:comment> Begin toc icon mapping </xsl:comment>
   <xsl:variable name="VarBehaviorsPath" select="$ParamBehaviorsByType[@documentID = $VarGroupDocument/@DocumentID]/@path" />
   <xsl:variable name="VarBehaviors" select="wwexsldoc:LoadXMLWithoutResolver($VarBehaviorsPath)" />
   <xsl:variable name="VarTOCIconsAsXML">
    <xsl:for-each select="$VarBehaviors/wwbehaviors:Behaviors//wwbehaviors:Marker[@behavior = 'toc-icon-java-help']">
     <xsl:variable name="VarMarker" select="." />
     <xsl:variable name="VarTOCIconText">
      <xsl:for-each select="$VarMarker//wwdoc:TextRun/wwdoc:Text">
       <xsl:value-of select="@value" />
      </xsl:for-each>
     </xsl:variable>

     <wwmap:Image path="{$VarTOCIconText}" />

    </xsl:for-each>
   </xsl:variable>
   <xsl:variable name="VarTOCIconTextAsNodeSet" select="msxsl:node-set($VarTOCIconsAsXML)" />

   <xsl:for-each select="$VarTOCIconTextAsNodeSet/wwmap:Image">
    <xsl:variable name="VarTOCIconPath" select="." />

    <xsl:variable name="VarTOCIconPathNodes" select="key('wwmap-image-by-path', $VarTOCIconPath/@path)" />
    <xsl:if test="count($VarTOCIconPathNodes[1] | $VarTOCIconPath) = 1">
     <xsl:variable name="VarTOCIconPathFileName" select="wwfilesystem:GetFileName($VarTOCIconPath/@path)" />
     <xsl:variable name="VarTOCIconMapID">
      <xsl:text>tocicon_</xsl:text>
      <xsl:value-of select="wwstring:ReplaceWithExpression($VarTOCIconPathFileName, '[^a-zA-Z0-9_]', '_')" />
     </xsl:variable>
     <javahelpmap:mapID target="{$VarTOCIconMapID}" url="{$VarTOCIconPath/@path}" />
    </xsl:if>
   </xsl:for-each>
   <xsl:comment> End toc icon mapping </xsl:comment>

  </xsl:for-each>

  <!-- Category icons: 'Table of Contents File' topic in JavaHelp 2.0 User's Guide -->
  <!--                                                                             -->
  <xsl:comment> Begin category images </xsl:comment>
  <xsl:variable name="VarCategoryClosedImage" select="wwprojext:GetFormatSetting('category-closed-image', '')" />
  <xsl:variable name="VarCategoryOpenImage" select="wwprojext:GetFormatSetting('category-open-image', '')" />

  <xsl:if test="string-length($VarCategoryClosedImage) &gt; 0">
   <javahelpmap:mapID target="category_closed_image" url="{$VarCategoryClosedImage}" />
  </xsl:if>

  <xsl:if test="string-length($VarCategoryOpenImage) &gt; 0">
   <javahelpmap:mapID target="category_open_image" url="{$VarCategoryOpenImage}" />
  </xsl:if>

  <xsl:comment> End category images </xsl:comment>

  <!-- Glossary files -->
  <!--                -->
  <xsl:if test="wwprojext:GetFormatSetting('enable-glossary-tab', 'false') = 'true'">
   <xsl:comment> Begin glossary entries </xsl:comment>
   <xsl:for-each select="$ParamGlossaryByType">
    <xsl:variable name="VarGlossaryFile" select="." />

    <xsl:variable name="VarGlossaryFilePath" select="$VarGlossaryFile/@path" />
    <xsl:variable name="VarTarget">
     <xsl:text>glossary_</xsl:text>
     <xsl:value-of select="wwstring:Replace(wwfilesystem:GetFileNameWithoutExtension($VarGlossaryFile/@path), ' ', '_')" />
    </xsl:variable>
    <javahelpmap:mapID target="{$VarTarget}" url="{wwuri:Unescape(wwuri:GetRelativeTo($VarGlossaryFilePath, $ParamMapPath))}" />
   </xsl:for-each>
   <xsl:comment> End glossary entries </xsl:comment>
  </xsl:if>
 </xsl:template>
</xsl:stylesheet>
